/**
 * 
 */
package com.materna.federatedcloudsim;

import java.util.ArrayList;
import java.util.List;

import org.cloudbus.cloudsim.Cloudlet;
import org.cloudbus.cloudsim.UtilizationModel;

import com.materna.federatedcloudsim.federation.ServiceRequest;
import com.materna.federatedcloudsim.monitoring.FileLog;

/**
 * @author Marc Spohr
 *
 */
public class FederatedCloudlet extends Cloudlet implements Comparable<FederatedCloudlet>
{
	private double eventTime;
	private ServiceRequest sr;
	private FederatedVm vm;
	private final List<FederatedVm> vmHistory;
	private static int nextId = 0;

	/**
	 * @param cloudletLength the length or size (in MI) of this cloudlet to be executed in a PowerDatacenter
	 * @param pesNumber the pes number
	 * @param cloudletFileSize the file size (in byte) of this cloudlet BEFORE submitting to a PowerDatacenter
	 * @param cloudletOutputSize the file size (in byte) of this cloudlet AFTER finish executing by a PowerDatacenter
	 * @param utilizationModelCpu the {@link UtilizationModel} cpu
	 * @param utilizationModelRam the {@link UtilizationModel} ram
	 * @param utilizationModelBw the {@link UtilizationModel} bw
	 * @param sr the service request this cloudlet belongs to
	 */
	public FederatedCloudlet(long cloudletLength,
			int pesNumber, long cloudletFileSize, long cloudletOutputSize,
			UtilizationModel utilizationModelCpu,
			UtilizationModel utilizationModelRam,
			UtilizationModel utilizationModelBw,
			ServiceRequest sr)
	{
		super(nextId++, cloudletLength, pesNumber, cloudletFileSize,
				cloudletOutputSize, utilizationModelCpu, utilizationModelRam,
				utilizationModelBw);
		vmHistory = new ArrayList<>();
		setSr(sr);
	}

	/**
	 * @param cloudletLength the length or size (in MI) of this cloudlet to be executed in a PowerDatacenter
	 * @param pesNumber the pes number
	 * @param cloudletFileSize the file size (in byte) of this cloudlet BEFORE submitting to a PowerDatacenter
	 * @param cloudletOutputSize the file size (in byte) of this cloudlet AFTER finish executing by a PowerDatacenter
	 * @param utilizationModelCpu the {@link UtilizationModel} cpu
	 * @param utilizationModelRam the {@link UtilizationModel} ram
	 * @param utilizationModelBw the {@link UtilizationModel} bw
	 * @param record record the history of this object or not
	 * @param fileList list of files required by this cloudlet
	 * @param sr the service request this cloudlet belongs to
	 */
	public FederatedCloudlet(long cloudletLength,
			int pesNumber, long cloudletFileSize, long cloudletOutputSize,
			UtilizationModel utilizationModelCpu,
			UtilizationModel utilizationModelRam,
			UtilizationModel utilizationModelBw, boolean record,
			List<String> fileList,
			ServiceRequest sr)
	{
		super(nextId++, cloudletLength, pesNumber, cloudletFileSize,
				cloudletOutputSize, utilizationModelCpu, utilizationModelRam,
				utilizationModelBw, record, fileList);
		vmHistory = new ArrayList<>();
		setSr(sr);
	}

	/**
	 * @param cloudletLength the length or size (in MI) of this cloudlet to be executed in a PowerDatacenter
	 * @param pesNumber the pes number
	 * @param cloudletFileSize the file size (in byte) of this cloudlet BEFORE submitting to a PowerDatacenter
	 * @param cloudletOutputSize the file size (in byte) of this cloudlet AFTER finish executing by a PowerDatacenter
	 * @param utilizationModelCpu the {@link UtilizationModel} cpu
	 * @param utilizationModelRam the {@link UtilizationModel} ram
	 * @param utilizationModelBw the {@link UtilizationModel} bw
	 * @param fileList list of files required by this cloudlet
	 * @param sr the service request this cloudlet belongs to
	 */
	public FederatedCloudlet(long cloudletLength,
			int pesNumber, long cloudletFileSize, long cloudletOutputSize,
			UtilizationModel utilizationModelCpu,
			UtilizationModel utilizationModelRam,
			UtilizationModel utilizationModelBw, List<String> fileList,
			ServiceRequest sr)
	{
		super(nextId++, cloudletLength, pesNumber, cloudletFileSize,
				cloudletOutputSize, utilizationModelCpu, utilizationModelRam,
				utilizationModelBw, fileList);
		vmHistory = new ArrayList<>();
		setSr(sr);
	}

	/**
	 * @param cloudletLength the length or size (in MI) of this cloudlet to be executed in a PowerDatacenter
	 * @param pesNumber the pes number
	 * @param cloudletFileSize the file size (in byte) of this cloudlet BEFORE submitting to a PowerDatacenter
	 * @param cloudletOutputSize the file size (in byte) of this cloudlet AFTER finish executing by a PowerDatacenter
	 * @param utilizationModelCpu the {@link UtilizationModel} cpu
	 * @param utilizationModelRam the {@link UtilizationModel} ram
	 * @param utilizationModelBw the {@link UtilizationModel} bw
	 * @param record record the history of this object or not
	 * @param sr the service request this cloudlet belongs to
	 */
	public FederatedCloudlet(long cloudletLength,
			int pesNumber, long cloudletFileSize, long cloudletOutputSize,
			UtilizationModel utilizationModelCpu,
			UtilizationModel utilizationModelRam,
			UtilizationModel utilizationModelBw, boolean record,
			ServiceRequest sr)
	{
		super(nextId++, cloudletLength, pesNumber, cloudletFileSize,
				cloudletOutputSize, utilizationModelCpu, utilizationModelRam,
				utilizationModelBw, record);
		vmHistory = new ArrayList<>();
		setSr(sr);
	}
	
	/**
	 * @return the Event time
	 */
	public double getEventTime()
	{
		return eventTime;
	}

	/**
	 * @param eventTime the Event time to set
	 */
	public void setEventTime(double eventTime)
	{
		this.eventTime = eventTime;
	}

	/**
	 * @return the {@link ServiceRequest}
	 */
	public ServiceRequest getSr()
	{
		return sr;
	}

	/**
	 * @param sr the {@link ServiceRequest} to set
	 */
	private void setSr(ServiceRequest sr)
	{
		this.sr = sr;
		if(getSr() != null)
			getSr().addCloudlet(this);
	}

	/**
	 * @return the {@link FederatedVm} this Cloudlet has been scheduled to
	 */
	public FederatedVm getVm()
	{
		return vm;
	}

	/**
	 * @param vm the {@link FederatedVm} to set
	 */
	public void setVm(FederatedVm vm)
	{
		if(getVm() != null) //Remove Cloudlet from old VM
		{
			getVm().removeCloudlet(this);
			super.setVmId(-1);
		}
		this.vm = vm;
		FileLog.getInstance().cl_schedule(this);
		if(getVm() != null) //Add Cloudlet to new VM
		{
			getVm().addCloudlet(this);
			getVmHistory().add(vm);
			super.setVmId(getVm().getId());
		}
	}

	/**
	 * @return the List of {@link FederatedVm} this cloudlet has previously been scheduled to
	 */
	public List<FederatedVm> getVmHistory()
	{
		return vmHistory;
	}

	@Override
	public int compareTo(FederatedCloudlet o) {
		return sr.compareTo(o.getSr());
	}
}
