/**
 * 
 */
package com.materna.federatedcloudsim;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

import org.cloudbus.cloudsim.Host;
import org.cloudbus.cloudsim.Pe;
import org.cloudbus.cloudsim.Vm;
import org.cloudbus.cloudsim.VmScheduler;
import org.cloudbus.cloudsim.provisioners.BwProvisioner;
import org.cloudbus.cloudsim.provisioners.RamProvisioner;

import com.materna.federatedcloudsim.federation.ServiceProviderRegistry;
import com.materna.federatedcloudsim.monitoring.ResourcesBean;
import com.materna.federatedcloudsim.monitoring.ResourcesBeanHost;
import com.materna.federatedcloudsim.provisioners.FederatedRamProvisionerOvercommitment;

/**
 * @author Marc Spohr
 *
 */
public class FederatedHost extends Host implements Comparable<FederatedHost> {
	private final ResourcesBean resources;
	private double powerConsumption;
	private boolean hostTurnedOn;
	private int state;
	private int statechangeTime = 5;

	/**
	 * @param id
	 *            the unique Id of this Host
	 * @param ramProvisioner
	 *            the {@link RamProvisioner}
	 * @param bwProvisioner
	 *            the {@link BwProvisioner}
	 * @param storage
	 *            amount of storage this host provides
	 * @param peList
	 *            the list of physical entities
	 * @param vmScheduler
	 *            the {@link VmScheduler} to use
	 */
	public FederatedHost(int id, RamProvisioner ramProvisioner, BwProvisioner bwProvisioner, long storage,
			List<? extends Pe> peList, VmScheduler vmScheduler) {
		super(id, ramProvisioner, bwProvisioner, storage, peList, vmScheduler);
		resources = new ResourcesBeanHost(this);
		powerConsumption = 0;
		hostTurnedOn = true;
		// host is activated at creation
		this.state = 0;
	}

//	@Override
//	public <T extends Vm> List<T> getVmList() {
//		List<Vm> list = super.getVmList();
//		return (List<T>) new List<Vm>() {
//
//			@Override
//			public boolean add(Vm e) {
//				// TODO: TESTOTESTOTESTITEST
//				for(CloudServiceProvider csp : ServiceProviderRegistry.getInstance().getCspList())
//					for(FederatedDatacenter fedDc : csp.getDcList())
//						for(FederatedHost fedHost : fedDc.getHostOnList()) {
//							if (fedHost == FederatedHost.this) continue;
//							if(fedHost.getVmList().contains(e))
//								System.out.println("PROBLEM");
//						}
//				
//				return list.add(e);
//			}
//
//			@Override
//			public void add(int index, Vm element) {
//				list.add(index, element);
//			}
//
//			@Override
//			public boolean addAll(Collection c) {
//				return list.addAll(c);
//			}
//
//			@Override
//			public boolean addAll(int index, Collection c) {
//				return list.addAll(index, c);
//			}
//
//			@Override
//			public void clear() {
//				list.clear();
//			}
//
//			@Override
//			public boolean contains(Object o) {
//				return list.contains(o);
//			}
//
//			@Override
//			public boolean containsAll(Collection c) {
//				return list.containsAll(c);
//			}
//
//			@Override
//			public Vm get(int index) {
//				return list.get(index);
//			}
//
//			@Override
//			public int indexOf(Object o) {
//				return list.indexOf(o);
//			}
//
//			@Override
//			public boolean isEmpty() {
//				return list.isEmpty();
//			}
//
//			@Override
//			public Iterator iterator() {
//				return list.iterator();
//			}
//
//			@Override
//			public int lastIndexOf(Object o) {
//				return lastIndexOf(o);
//			}
//
//			@Override
//			public ListIterator listIterator() {
//				return list.listIterator();
//			}
//
//			@Override
//			public ListIterator listIterator(int index) {
//				return list.listIterator(index);
//			}
//
//			@Override
//			public boolean remove(Object o) {
//				return list.remove(o);
//			}
//
//			@Override
//			public Vm remove(int index) {
//				return list.remove(index);
//			}
//
//			@Override
//			public boolean removeAll(Collection c) {
//				return list.removeAll(c);
//			}
//
//			@Override
//			public boolean retainAll(Collection c) {
//				return list.retainAll(c);
//			}
//
//			@Override
//			public Vm set(int index, Vm element) {
//				return list.set(index, element);
//			}
//
//			@Override
//			public int size() {
//				return list.size();
//			}
//
//			@Override
//			public List subList(int fromIndex, int toIndex) {
//				return list.subList(fromIndex, toIndex);
//			}
//
//			@Override
//			public Object[] toArray() {
//				return list.toArray();
//			}
//
//			@Override
//			public Object[] toArray(Object[] a) {
//				return list.toArray(a);
//			}
//
//		};
//	}

	/**
	 * @return number of cloudlets over all vms on this host
	 */
	public int getCloudletCount() {
		int cloudlets = 0;
		for (Vm vm : getVmList())
			cloudlets += (vm instanceof FederatedVm ? ((FederatedVm) vm).getCloudlets().size() : 0);
		return cloudlets;
	}

	/**
	 * @return the used storage over all vms on this host
	 */
	public long getUsedStorage() {
		long storage = 0;
		for (Vm vm : getVmList())
			storage += vm.getSize();
		return storage;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.cloudbus.cloudsim.Host#getDatacenter()
	 */
	@Override
	public FederatedDatacenter getDatacenter() {
		return (FederatedDatacenter) super.getDatacenter();
	}

	/**
	 * @return the resources
	 */
	public ResourcesBean getResources() {
		return resources;
	}

	public double getOvercommitValue() {

		double configuredRam = 0;
		for (Vm vm : getVmList()) {
			configuredRam += vm.getRam();
		}

		return configuredRam / getRam();
	}

	/**
	 * @return the value of hostTurnedOn.
	 * @author dpastern
	 */
	public boolean getHostTurnedOn() {
		return this.hostTurnedOn;
	}

	/**
	 * @author dpastern
	 * @pre no VMs running on host
	 */

	public boolean shutdown() {
		if (this.getVmList().size() == 0) {
			this.hostTurnedOn = false;
			return true;
		} else {
			return false;
		}
	}

	/**
	 * @author dpastern
	 */

	public boolean start() {
		if (this.hostTurnedOn) {
			return false;
		} else {
			this.hostTurnedOn = true;
			return true;
		}
	}

	/**
	 * @author dpastern
	 * @return the powerconsumption of the host
	 */

	public double getPowerconsumtion() {
		return this.powerConsumption;
	}

	public int getStatechangeTime() {
		return this.statechangeTime;
	}

	public int getState() {
		return this.state;
	}

	public void setState(int state) {
		this.state = state;
	}

	/**
	 * Checks if is suitable for vm based on threshold values.
	 * 
	 * @param vm
	 *            the vm
	 * @return true, if is suitable for vm
	 */

	public boolean isSuitableForVm(FederatedVm vm, float th) {
		return (state == 0 && vm.getResources().getPeCount().getTotal() <= this.getNumberOfPes()
				&& getResources().getMips().getTotal() * th >= vm.getResources().getMips().getUsed()
						+ getResources().getMips().getUsed()
				&& getResources().getRam().getTotal() * th >= vm.getResources().getRam().getUsed()
						+ getResources().getRam().getUsed()
				// && getRamProvisioner().isSuitableForVm(vm,
				// vm.getResources().getRam().getUsed())
				// TODO angepasst
				&& getRamProvisioner().isSuitableForVm(vm, vm.getResources().getRam().getTotal())
				&& getBwProvisioner().isSuitableForVm(vm, vm.getCurrentRequestedBw()));
	}

	public boolean isSuitableForVmOFF(float th, double mips, int ram, int allocRam, int peCount, long bw) {
		return (peCount <= this.getNumberOfPes()

				&& getResources().getMips().getTotal() * th >= mips

				&& getResources().getRam().getTotal() * th >= ram

				&& allocRam / ((FederatedRamProvisionerOvercommitment) getRamProvisioner())
						.getRam() <= ((FederatedRamProvisionerOvercommitment) getRamProvisioner()).getMaxOcValue()

				&& getBwProvisioner().getAvailableBw() > bw);
	}

	public double getCpuPerc() {
		return getResources().getMips().getUsed() / getResources().getMips().getTotal();
	}

	public double getRamPerc() {
		return (double) getResources().getRam().getUsed() / (double) getResources().getRam().getTotal();
	}

	// TODO new because of missing cloudletScheduler
	// TODO check MIPS if they were set correctly in the workload file
	@Override
	public boolean isSuitableForVm(Vm vm) {
		boolean ret = false;

		if (vm.getCloudletScheduler() != null)
			ret = super.isSuitableForVm(vm);
		else
			ret = getRamProvisioner().isSuitableForVm(vm, vm.getCurrentRequestedRam())
					&& getBwProvisioner().isSuitableForVm(vm, vm.getCurrentRequestedBw());

		return ret && state == 0;
	}

	@Override
	public int compareTo(FederatedHost o) {
		int erg = 0;
		String ergCacheThis = null;
		String ergCacheO = null;

		ergCacheThis = getDatacenter().getId() + "::" + getId();
		ergCacheO = o.getDatacenter().getId() + "::" + o.getId();

		erg = ergCacheThis.compareTo(ergCacheO);

		return erg;
	}

}
