package com.materna.federatedcloudsim;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

import org.cloudbus.cloudsim.UtilizationModel;

import com.materna.federatedcloudsim.core.FederatedConfigurationException;

/**
 * The Class UtilizationModelBitbrains.
 * 
 * @author Damian Pasternak
 */
public class UtilizationModelBitbrains implements UtilizationModel {

	/** The data (5 min * 8064 = 28 days). */
	private final double[] data;

	/**
	 * Instantiates a new utilization model BitBrains.
	 * 
	 * @param inputPath
	 *            the input path
	 * @param modifier
	 *            1 select CPU utilization, 2 select RAM utilization
	 * @throws NumberFormatException
	 *             the number format exception
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public UtilizationModelBitbrains(String inputPath, int modifier, int size)
			throws NumberFormatException, IOException {

		data = new double[size];
		String[] logData;
		BufferedReader input = new BufferedReader(new FileReader(inputPath));
		input.readLine(); // skip first line in file
		int n = data.length;
		for (int i = 0; i < n; i++) {
			String line = input.readLine();
			if (line != null) {
				logData = line.split(";");
				data[i] = Double.valueOf(logData[modifier]);
			} else
				data[i] = 0.0;
		}
		input.close();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see cloudsim.power.UtilizationModel#getUtilization(double)
	 */
	@Override
	public double getUtilization(double time) {
		if (((int) Math.floor(time / 300)) < data.length)
			return data[(int) Math.floor(time / 300)];
		else if (((int) Math.floor(time / 300)) == data.length)
			return 0.0;
		else
			throw new FederatedConfigurationException("Simulation endtime is bigger then \"utilization.size\"");

	}

}