/**
 * 
 */
package com.materna.federatedcloudsim.core;

import java.util.Calendar;
import java.util.Iterator;

import org.cloudbus.cloudsim.core.CloudSim;
import org.cloudbus.cloudsim.core.SimEvent;

import com.materna.federatedcloudsim.monitoring.FileLog;
import com.materna.federatedcloudsim.monitoring.Monitoring;
import com.materna.federatedcloudsim.utils.CloudSimConfigurator;

/**
 * @author Marc Spohr
 *
 */
public class FederatedCloudSim extends CloudSim
{
	private final CloudSimConfigurator config;
	
	/**
	 * @param propFile File Name of Properties file ...
	 */
	public FederatedCloudSim(String propFile)
	{
//		config = new CloudSimConfigurator(propFile);
		config = CloudSimConfigurator.createInstance(propFile);
		
		if(config.configureCloudSim())
		{
			startSimulation();
		}
	}

	/**
	 * @param args the commandline arguments
	 */
	public static void main(String[] args)
	{
		if(args.length == 0)
			new FederatedCloudSim(null);
		else
			for(String propFile : args)
				new FederatedCloudSim(propFile);
	}

	/**
	 * starts Resource Monitoring, calls {@link CloudSim#init} and replaces FutureQueue with Resource-Logging Version<br/>
	 * <b>ensure to call this Method instead of {@link CloudSim#init} in order to have Resource Monitoring activated!!</b>
	 * @see CloudSim#init(int, Calendar, boolean)
	 */
	public static void init(int numUser, Calendar cal, boolean traceFlag)
	{
		//initialize CloudSim
		CloudSim.init(numUser, cal, traceFlag);

		//Initiate Resource Monitoring Singleton
		Monitoring.getInstance();
		
		//exchange FutureQueue and copy existing elements to new queue
		MonitoringFutureQueue monFuture = new MonitoringFutureQueue();
		Iterator<SimEvent> fiterator = future.iterator();
		while(fiterator.hasNext())
		{
			monFuture.addEvent(fiterator.next());
		}
		future = monFuture;
	}
	
	/**
	 * calls {@link CloudSim#startSimulation()} and writes Summary-Log
	 * @see CloudSim#startSimulation()
	 */
	public static double startSimulation() throws NullPointerException
	{
		double retVal = CloudSim.startSimulation();
		FileLog.getInstance().summary();
		return retVal;
	}
}
