/**
 * 
 */
package com.materna.federatedcloudsim.federation;

import java.util.Set;
import java.util.TreeSet;

import com.materna.federatedcloudsim.FederatedCloudlet;
import com.materna.federatedcloudsim.federation.sla.Sla;

/**
 * @author Marc Spohr
 *
 */
public class ServiceRequest implements Comparable<ServiceRequest>
{
	private static int nextId = 0;
	private final int id;
	private final Set<FederatedCloudlet> cloudletList;
	private final Set<FederatedCloudlet> cloudletFinishedList;
	private Sla sla = null;
	private double requestTime;
	private int jobNr;

	
	/**
	 * 
	 */
	public ServiceRequest()
	{
		id = nextId++;
		cloudletList = new TreeSet<>();
		cloudletFinishedList = new TreeSet<>();
		jobNr = -1;
	}

	/**
	 * @param cloudlet the {@link FederatedCloudlet}
	 * @return {@code true} if cloudlet finished execution
	 */
	public boolean isCloudletFinished(FederatedCloudlet cloudlet)
	{
		if(getCloudletList().contains(cloudlet))
		{
			return getCloudletFinishedList().add(cloudlet);
		}
		else return false;
	}
	
	/**
	 * @return {@code true} if this ServiceRequest finished execution (all cloudlets finished}
	 */
	public boolean isFinished()
	{
		return getCloudletFinishedList().containsAll(getCloudletList());
	}

	
	/**
	 * @return the {@link Sla}
	 */
	public Sla getSla()
	{
		return sla;
	}

	/**
	 * @param sla the {@link Sla} to set
	 */
	public void setSla(Sla sla)
	{
		this.sla = sla;
	}

	/**
	 * @return the id
	 */
	public int getId()
	{
		return id;
	}

	/**
	 * @return the requestTime
	 */
	public double getRequestTime()
	{
		return requestTime;
	}

	/**
	 * @param requestTime the requestTime to set
	 */
	public void setRequestTime(double requestTime)
	{
		this.requestTime = requestTime;
	}

	/**
	 * @param cloudlet the {@link FederatedCloudlet} to add
	 */
	public void addCloudlet(FederatedCloudlet cloudlet)
	{
		getCloudletList().add(cloudlet);
	}
	
	/**
	 * @return the Set of {@link FederatedCloudlet} this ServiceRequest contains
	 */
	public Set<FederatedCloudlet> getCloudletList()
	{
		return cloudletList;
	}

	/**
	 * @return the Set of {@link FederatedCloudlet} finished their execution
	 */
	public Set<FederatedCloudlet> getCloudletFinishedList()
	{
		return cloudletFinishedList;
	}

	public void setJobNr(int jobNr) {
		this.jobNr = jobNr;
	}
	
	public int getJobNr() {
		return jobNr;
	}

	@Override
	public int compareTo(ServiceRequest o) {
		int erg = 0;
		
		if (getId() < o.getId())
			erg = -1;
		else if (getId() == o.getId())
			erg = 0;
		else if (getId() > o.getId())
			erg = 1;
		return erg;
	}
}
