package com.materna.federatedcloudsim.lists;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.cloudbus.cloudsim.Vm;
import org.cloudbus.cloudsim.lists.VmList;

import com.materna.federatedcloudsim.FederatedHost;
import com.materna.federatedcloudsim.FederatedVm;

public class FederatedHostList extends VmList {

	/**
	 * Sort by mips use.
	 * 
	 * @param vmList
	 *            the vm list
	 */
	public static <T extends FederatedHost> void sortDescendingByUsedMips(List<T> hostList) {

		Comparator<T> comparator = new Comparator<T>() {
			@Override
			public int compare(T a, T b) throws ClassCastException {
				Double aUsed = a.getResources().getMips().getUsed();
				Double bUsed = b.getResources().getMips().getUsed();
				return bUsed.compareTo(aUsed);
			}
		};

		Collections.sort(hostList, comparator);
	}
	
	public static <T extends FederatedHost> void sortAscendingByUsedMips(List<T> hostList) {

		Comparator<T> comparator = new Comparator<T>() {
			@Override
			public int compare(T a, T b) throws ClassCastException {
				Double aUsed = a.getResources().getMips().getUsed();
				Double bUsed = b.getResources().getMips().getUsed();
				return aUsed.compareTo(bUsed);
			}
		};

		Collections.sort(hostList, comparator);
	}
	
	public static <T extends FederatedHost> void sortDescendingById(List<T> hostList) {

		Comparator<T> comparator = new Comparator<T>() {
			@Override
			public int compare(T a, T b) throws ClassCastException {
				Integer aId = a.getId();
				Integer bId = b.getId();
				return aId.compareTo(bId);
			}
		};

		Collections.sort(hostList, comparator);
	}

	/**
	 * Sort by ram use.
	 * 
	 * @param vmList
	 *            the vm list
	 */
	public static <T extends FederatedHost> void sortAscendingByFreeRam(List<T> hostList) {

		Comparator<T> comparator = new Comparator<T>() {
			@Override
			public int compare(T a, T b) throws ClassCastException {
				Integer aFree = a.getResources().getRam().getTotal() - a.getResources().getRam().getUsed();
				Integer bFree = b.getResources().getRam().getTotal() - b.getResources().getRam().getUsed();
				return Integer.valueOf(aFree.compareTo(bFree));
			}
		};

		Collections.sort(hostList, comparator);
	}
	
	
	public static <T extends FederatedHost> void sortDescendingByFreeRam(List<T> hostList) {

		Comparator<T> comparator = new Comparator<T>() {
			@Override
			public int compare(T a, T b) throws ClassCastException {
				Integer aFree = a.getResources().getRam().getTotal() - a.getResources().getRam().getUsed();
				Integer bFree = b.getResources().getRam().getTotal() - b.getResources().getRam().getUsed();
				return Integer.valueOf(bFree.compareTo(aFree));
			}
		};

		Collections.sort(hostList, comparator);
	}

	public static <T extends FederatedHost> void sortAscendingByOcValue(List<T> hostList) {

		Comparator<T> comparator = new Comparator<T>() {
			@Override
			public int compare(T a, T b) throws ClassCastException {

				int aConfigured = 0;
				int bConfigured = 0;
				for (Vm vm : a.getVmList()) {
					aConfigured = aConfigured + vm.getRam();
				}
				for (Vm vm : b.getVmList()) {
					bConfigured = bConfigured + vm.getRam();
				}

				Integer aFree = aConfigured / a.getResources().getRam().getTotal();
				Integer bFree = bConfigured / b.getResources().getRam().getTotal();
				return Integer.valueOf(aFree.compareTo(bFree));
			}
		};

		Collections.sort(hostList, comparator);
	}


}
