package com.materna.federatedcloudsim.monitoring;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.cloudbus.cloudsim.Host;
import org.cloudbus.cloudsim.Log;
import org.cloudbus.cloudsim.core.CloudSim;
import org.cloudbus.cloudsim.core.CloudSimTags;
import org.cloudbus.cloudsim.core.SimEntity;
import org.cloudbus.cloudsim.core.SimEvent;

import com.materna.federatedcloudsim.CloudServiceProvider;
import com.materna.federatedcloudsim.FederatedDatacenter;
import com.materna.federatedcloudsim.FederatedHost;
import com.materna.federatedcloudsim.FederatedVm;
import com.materna.federatedcloudsim.core.FederatedCloudSimTags;
import com.materna.federatedcloudsim.federation.ServiceProviderRegistry;
import com.materna.federatedcloudsim.federation.sla.Sla;

/**
 * 
 * @author Marco Pfahlberg
 *
 */
public class Endlog extends SimEntity {

	private static Endlog instance;

	private NumberFormat nf = NumberFormat.getInstance();

	private long intraDcMigCount;
	private long intraCspMigCount;
	private long intraFedMigCount = 0;
	private long intraFedMigCount_old;
	private int simEndtime = 0;

	private Map<FederatedDatacenter, Long> intraDcMigrationCount;
	private Map<CloudServiceProvider, Long> intraCspMigrationCount;
	private Map<CloudServiceProvider, Long> outgoingMigrationCountCsp_ownVm = new HashMap<CloudServiceProvider, Long>();
	private Map<CloudServiceProvider, Long> incomingMigrationCountCsp_ownVm = new HashMap<CloudServiceProvider, Long>();
	private Map<CloudServiceProvider, Long> outgoingMigrationCountCsp_foreignVm = new HashMap<CloudServiceProvider, Long>();
	private Map<CloudServiceProvider, Long> incomingMigrationCountCsp_foreignVm = new HashMap<CloudServiceProvider, Long>();

	private Map<FederatedDatacenter, Map<FederatedDatacenter, Long>> outgoingMigrationsCountDc = new HashMap<FederatedDatacenter, Map<FederatedDatacenter, Long>>();
	private Map<FederatedDatacenter, Map<FederatedDatacenter, Long>> incomingMigrationsCountDc = new HashMap<FederatedDatacenter, Map<FederatedDatacenter, Long>>();

	private Map<FederatedHost, Map<FederatedHost, Long>> outgoingMigrationsCountHost = new HashMap<FederatedHost, Map<FederatedHost, Long>>();
	private Map<FederatedHost, Map<FederatedHost, Long>> incomingMigrationsCountHost = new HashMap<FederatedHost, Map<FederatedHost, Long>>();

	private Map<FederatedDatacenter, Integer> slaWarningsCount_csp = new HashMap<FederatedDatacenter, Integer>();
	private Map<FederatedDatacenter, Integer> ramThWarningCount_csp = new HashMap<FederatedDatacenter, Integer>();
	private Map<FederatedDatacenter, Integer> ramDangerThWarningCount_csp = new HashMap<FederatedDatacenter, Integer>();
	private Map<FederatedDatacenter, Integer> cpuThWarningCount_csp = new HashMap<FederatedDatacenter, Integer>();
	private Map<FederatedDatacenter, Integer> cpuDangerThWarningCount_csp = new HashMap<FederatedDatacenter, Integer>();
	private Map<FederatedDatacenter, Long> consolidationCallCount_csp = new HashMap<FederatedDatacenter, Long>();

	private String federationBrokenRamString = "";
	private String federationBrokenCpuString = "";
	private Map<CloudServiceProvider, String> cspBrokenRamString = new HashMap<CloudServiceProvider, String>();
	private Map<CloudServiceProvider, String> cspBrokenCpuString = new HashMap<CloudServiceProvider, String>();
	private Map<FederatedDatacenter, String> dcBrokenRamString = new HashMap<FederatedDatacenter, String>();
	private Map<FederatedDatacenter, String> dcBrokenCpuString = new HashMap<FederatedDatacenter, String>();

	private Map<FederatedDatacenter, Map<FederatedVm, Integer>> cpuBrokenCount_csp = new HashMap<FederatedDatacenter, Map<FederatedVm, Integer>>();
	private Map<FederatedDatacenter, Map<FederatedVm, Integer>> ramBrokenCount_csp = new HashMap<FederatedDatacenter, Map<FederatedVm, Integer>>();

	/*
	 * Mean values
	 */

	private Map<CloudServiceProvider, Double> meanValuesCpuAct_CSP;
	private Map<CloudServiceProvider, Integer> meanValuesRamAct_CSP;
	private Map<FederatedDatacenter, Double> meanValuesCpuAct_DC;
	private Map<FederatedDatacenter, Integer> meanValuesRamAct_DC;

	// overflowPrevention
	private Map<CloudServiceProvider, List<Double>> meanValuesCpu_CSP;
	private Map<CloudServiceProvider, List<Integer>> meanValuesRam_CSP;
	private Map<FederatedDatacenter, List<Double>> meanValuesCpu_DC;
	private Map<FederatedDatacenter, List<Integer>> meanValuesRam_DC;

	private Map<CloudServiceProvider, List<Long>> mean_Count_Cpu_CSP;
	private Map<CloudServiceProvider, List<Long>> mean_Count_Ram_CSP;
	private Map<FederatedDatacenter, List<Long>> mean_Count_Cpu_DC;
	private Map<FederatedDatacenter, List<Long>> mean_Count_Ram_DC;

	private long meanActCountRam_DC = 0;
	private long meanActCountCpu_DC = 0;
	private long meanActCountRam_CSP = 0;
	private long meanActCountCpu_CSP = 0;

	private long meanCount = 0;

	private Map<FederatedDatacenter, Double> maxCpuDC = new HashMap<FederatedDatacenter, Double>();
	private Map<FederatedDatacenter, Integer> maxRamDC = new HashMap<FederatedDatacenter, Integer>();
	private Map<CloudServiceProvider, Double> maxCpuCSP = new HashMap<CloudServiceProvider, Double>();
	private Map<CloudServiceProvider, Integer> maxRamCSP = new HashMap<CloudServiceProvider, Integer>();

	private String regionalCisName;

	private String simConfigName;
	private long simStart;
	private long simEnd;

	private long hostOnCounter = 0;
	private long hostTotalCounter = 0;

	private Map<FederatedDatacenter, Long> intraDCMigrationCount = new HashMap<FederatedDatacenter, Long>();

	private Map<CloudServiceProvider, Map<FederatedVm, Long>> migrationsVmCache = new HashMap<CloudServiceProvider, Map<FederatedVm, Long>>();

	private String configName = "";
	private int pingPong = 0;

	private Map<CloudServiceProvider, List<Integer>> hostsOn = new HashMap<CloudServiceProvider, List<Integer>>();
	private long hostOnCount = 0;
	private long hostOffCount = 0;

	private Map<CloudServiceProvider, Long> pingPongAvoidanceCount = new HashMap<CloudServiceProvider, Long>();
	private int ocValue = 0;

	private Map<CloudServiceProvider, Integer> cspTotalBrokenRam = new LinkedHashMap<CloudServiceProvider, Integer>();
	private Map<CloudServiceProvider, Integer> cspTotalBrokenCpu = new LinkedHashMap<CloudServiceProvider, Integer>();

	/*
	 * Overflow prevention while calculating meanLoads of DCs and CPSs False ->
	 * normal meanLoad; True -> weighted and cached meanLoads
	 */
	private boolean ofPrevention = false;

	private Endlog() {
		super("Endlog");
		intraDcMigCount = 0;
		intraCspMigCount = 0;
		intraFedMigCount_old = 0;

		intraDcMigrationCount = new HashMap<FederatedDatacenter, Long>();
		intraCspMigrationCount = new HashMap<CloudServiceProvider, Long>();
		incomingMigrationCountCsp_ownVm = new HashMap<CloudServiceProvider, Long>();

		meanValuesCpuAct_CSP = new HashMap<CloudServiceProvider, Double>();
		meanValuesRamAct_CSP = new HashMap<CloudServiceProvider, Integer>();
		meanValuesCpuAct_DC = new HashMap<FederatedDatacenter, Double>();
		meanValuesRamAct_DC = new HashMap<FederatedDatacenter, Integer>();

		meanValuesCpu_CSP = new HashMap<CloudServiceProvider, List<Double>>();
		meanValuesRam_CSP = new HashMap<CloudServiceProvider, List<Integer>>();
		meanValuesCpu_DC = new HashMap<FederatedDatacenter, List<Double>>();
		meanValuesRam_DC = new HashMap<FederatedDatacenter, List<Integer>>();

		mean_Count_Cpu_CSP = new HashMap<CloudServiceProvider, List<Long>>();
		mean_Count_Ram_CSP = new HashMap<CloudServiceProvider, List<Long>>();
		mean_Count_Cpu_DC = new HashMap<FederatedDatacenter, List<Long>>();
		mean_Count_Ram_DC = new HashMap<FederatedDatacenter, List<Long>>();
		initializeMeanLoadCache();

		simStart = 0;
		simEnd = 0;

		Set<CloudServiceProvider> cspSet = ServiceProviderRegistry.getInstance().getCspList();
		for (CloudServiceProvider csp : cspSet) {
			outgoingMigrationCountCsp_ownVm.put(csp, 0l);
			incomingMigrationCountCsp_ownVm.put(csp, 0l);
			migrationsVmCache.put(csp, new HashMap<FederatedVm, Long>());

			Set<FederatedDatacenter> fedDcSet = csp.getDcList();
			for (FederatedDatacenter fedDc : fedDcSet)
				intraDCMigrationCount.put(fedDc, 0l);

		}

		nf.setMaximumFractionDigits(4);
	}

	private void initializeMeanLoadCache() {
		for (CloudServiceProvider csp : ServiceProviderRegistry.getInstance().getCspList()) {
			meanValuesCpu_CSP.put(csp, new ArrayList<Double>());
			meanValuesRam_CSP.put(csp, new ArrayList<Integer>());
			mean_Count_Cpu_CSP.put(csp, new ArrayList<Long>());
			mean_Count_Ram_CSP.put(csp, new ArrayList<Long>());

			for (FederatedDatacenter fedDc : csp.getDcList()) {
				meanValuesCpu_DC.put(fedDc, new ArrayList<Double>());
				meanValuesRam_DC.put(fedDc, new ArrayList<Integer>());
				mean_Count_Cpu_DC.put(fedDc, new ArrayList<Long>());
				mean_Count_Ram_DC.put(fedDc, new ArrayList<Long>());
			}
		}
	}

	public static Endlog getInstance() {
		if (instance == null)
			instance = new Endlog();

		return instance;
	}

	/*
	 * 
	 * ***** MIGRATION COUNTS *****
	 * 
	 */

	// ADD Mig-Count

	public void addIntraDcMigCount() {
		this.intraDcMigCount++;
	}

	public void addIntraCspMigCount() {
		this.intraCspMigCount++;
	}

	public void addIntraFedMigCount() {
		this.intraFedMigCount_old++;
	}

	// GET Mig-Count

	public long getIntraCspMigCount() {
		return intraCspMigCount;
	}

	public long getIntraFedMigCount() {
		return intraFedMigCount_old;
	}

	public long getIntraDcMigCount() {
		return intraDcMigCount;
	}

	/*
	 * 
	 * ***** BROKEN COUNTS *****
	 * 
	 */

	// ADD Count

	public void addRamBrokenCount_csp(FederatedDatacenter fedDc, FederatedVm fedVm) {
		Map<FederatedVm, Integer> cache = null;

		if (ramBrokenCount_csp.containsKey(fedDc) && (cache = ramBrokenCount_csp.get(fedDc)) != null) {

			if (cache.containsKey(fedVm))
				cache.put(fedVm, cache.get(fedVm) + 1);
			else

				cache.put(fedVm, 1);

			ramBrokenCount_csp.put(fedDc, cache); // TODO redundant!?!

		} else {
			Map<FederatedVm, Integer> newCache = new HashMap<FederatedVm, Integer>();
			newCache.put(fedVm, 1);

			ramBrokenCount_csp.put(fedDc, newCache);
		}
	}

	public void addCpuBrokenCount_csp(FederatedDatacenter fedDc, FederatedVm fedVm) {
		Map<FederatedVm, Integer> cache = null;

		if (cpuBrokenCount_csp.containsKey(fedDc) && (cache = cpuBrokenCount_csp.get(fedDc)) != null) {

			if (cache.containsKey(fedVm))
				cache.put(fedVm, cache.get(fedVm) + 1);
			else
				cache.put(fedVm, 1);

			cpuBrokenCount_csp.put(fedDc, cache); // TODO redundant!?!

		} else {
			Map<FederatedVm, Integer> newCache = new HashMap<FederatedVm, Integer>();
			newCache.put(fedVm, 1);

			cpuBrokenCount_csp.put(fedDc, newCache);
		}
	}

	public String getCpuBrokenCountString_csp() {
		int total = 0;
		int vm = 0;
		int totalGold = 0;
		int vmGold = 0;
		int totalSilver = 0;
		int vmSilver = 0;
		int totalBronze = 0;
		int vmBronze = 0;

		Map<FederatedVm, Integer> cache = null;

		for (FederatedDatacenter fedDc : cpuBrokenCount_csp.keySet()) {

			if ((cache = cpuBrokenCount_csp.get(fedDc)) != null) {

				for (FederatedVm fedVm : cache.keySet()) {

					total += cache.get(fedVm);
					vm += 1;

					if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_GOLD) {
						totalGold += cache.get(fedVm);
						vmGold += 1;
					} else if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_SILVER) {
						totalSilver += cache.get(fedVm);
						vmSilver += 1;
					} else if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_BRONZE) {
						totalBronze += cache.get(fedVm);
						vmBronze += 1;
					}
				}
			}
		}

		return total + "/" + vm + "," + totalGold + "/" + vmGold + "," + totalSilver + "/" + vmSilver + ","
				+ totalBronze + "/" + vmBronze;
	}

	public String getRamBrokenCountString_csp() {
		int total = 0;
		int vm = 0;
		int totalGold = 0;
		int vmGold = 0;
		int totalSilver = 0;
		int vmSilver = 0;
		int totalBronze = 0;
		int vmBronze = 0;

		Map<FederatedVm, Integer> cache = null;

		for (FederatedDatacenter fedDc : cpuBrokenCount_csp.keySet()) {

			if ((cache = ramBrokenCount_csp.get(fedDc)) != null) {

				for (FederatedVm fedVm : cache.keySet()) {

					total += cache.get(fedVm);
					vm += 1;

					if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_GOLD) {
						totalGold += cache.get(fedVm);
						vmGold += 1;
					} else if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_SILVER) {
						totalSilver += cache.get(fedVm);
						vmSilver += 1;
					} else if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_BRONZE) {
						totalBronze += cache.get(fedVm);
						vmBronze += 1;
					}
				}
			}
		}

		return total + "/" + vm + "," + totalGold + "/" + vmGold + "," + totalSilver + "/" + vmSilver + ","
				+ totalBronze + "/" + vmBronze;
	}

	protected void generateBrokenStrings() {
		int totalRam = 0;
		int vmRam = 0;
		int totalRamGold = 0;
		int vmRamGold = 0;
		int totalRamSilver = 0;
		int vmRamSilver = 0;
		int totalRamBronze = 0;
		int vmRamBronze = 0;

		int totalCpu = 0;
		int vmCpu = 0;
		int totalCpuGold = 0;
		int vmCpuGold = 0;
		int totalCpuSilver = 0;
		int vmCpuSilver = 0;
		int totalCpuBronze = 0;
		int vmCpuBronze = 0;

		Set<CloudServiceProvider> cspSet = ServiceProviderRegistry.getInstance().getCspList();

		for (CloudServiceProvider csp : cspSet) {

			int totalRam_csp = 0;
			int vmRam_csp = 0;
			int totalRamGold_csp = 0;
			int vmRamGold_csp = 0;
			int totalRamSilver_csp = 0;
			int vmRamSilver_csp = 0;
			int totalRamBronze_csp = 0;
			int vmRamBronze_csp = 0;

			int totalCpu_csp = 0;
			int vmCpu_csp = 0;
			int totalCpuGold_csp = 0;
			int vmCpuGold_csp = 0;
			int totalCpuSilver_csp = 0;
			int vmCpuSilver_csp = 0;
			int totalCpuBronze_csp = 0;
			int vmCpuBronze_csp = 0;

			for (FederatedDatacenter fedDc : csp.getDcList()) {
				Map<FederatedVm, Integer> cacheRam = null;
				Map<FederatedVm, Integer> cacheCpu = null;

				if ((cacheRam = ramBrokenCount_csp.get(fedDc)) != null) {

					int total_dc = 0;
					int vm_dc = 0;
					int totalGold_dc = 0;
					int vmGold_dc = 0;
					int totalSilver_dc = 0;
					int vmSilver_dc = 0;
					int totalBronze_dc = 0;
					int vmBronze_dc = 0;

					if (csp.getDcList().contains(fedDc)) {

						for (FederatedVm fedVm : cacheRam.keySet()) {

							totalRam += cacheRam.get(fedVm);
							vmRam += 1;
							totalRam_csp += cacheRam.get(fedVm);
							vmRam_csp += 1;
							total_dc += cacheRam.get(fedVm);
							vm_dc += 1;

							if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_GOLD) {
								totalRamGold += cacheRam.get(fedVm);
								vmRamGold += 1;
								totalRamGold_csp += cacheRam.get(fedVm);
								vmRamGold_csp += 1;
								totalGold_dc += cacheRam.get(fedVm);
								vmGold_dc += 1;
							} else if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_SILVER) {
								totalRamSilver += cacheRam.get(fedVm);
								vmRamSilver += 1;
								totalRamSilver_csp += cacheRam.get(fedVm);
								vmRamSilver_csp += 1;
								totalSilver_dc += cacheRam.get(fedVm);
								vmSilver_dc += 1;
							} else if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_BRONZE) {
								totalRamBronze += cacheRam.get(fedVm);
								vmRamBronze += 1;
								totalRamBronze_csp += cacheRam.get(fedVm);
								vmRamBronze_csp += 1;
								totalBronze_dc += cacheRam.get(fedVm);
								vmBronze_dc += 1;
							}
						}

					} else {
						for (FederatedVm fedVm : cacheRam.keySet()) {

							totalRam += cacheRam.get(fedVm);
							vmRam += 1;
							total_dc += cacheRam.get(fedVm);
							vm_dc += 1;

							if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_GOLD) {
								totalRamGold += cacheRam.get(fedVm);
								vmRamGold += 1;
								totalGold_dc += cacheRam.get(fedVm);
								vmGold_dc += 1;
							} else if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_SILVER) {
								totalRamSilver += cacheRam.get(fedVm);
								vmRamSilver += 1;
								totalSilver_dc += cacheRam.get(fedVm);
								vmSilver_dc += 1;
							} else if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_BRONZE) {
								totalRamBronze += cacheRam.get(fedVm);
								vmRamBronze += 1;
								totalBronze_dc += cacheRam.get(fedVm);
								vmBronze_dc += 1;
							}
						}
					}

					dcBrokenRamString.put(fedDc, total_dc + "/" + vm_dc + "," + totalGold_dc + "/" + vmGold_dc + ","
							+ totalSilver_dc + "/" + vmSilver_dc + "," + totalBronze_dc + "/" + vmBronze_dc);
				}

				if ((cacheCpu = cpuBrokenCount_csp.get(fedDc)) != null) {

					int total_dc = 0;
					int vm_dc = 0;
					int totalGold_dc = 0;
					int vmGold_dc = 0;
					int totalSilver_dc = 0;
					int vmSilver_dc = 0;
					int totalBronze_dc = 0;
					int vmBronze_dc = 0;

					if (csp.getDcList().contains(fedDc)) {

						for (FederatedVm fedVm : cacheCpu.keySet()) {

							totalCpu += cacheCpu.get(fedVm);
							vmCpu += 1;
							totalCpu_csp += cacheCpu.get(fedVm);
							vmCpu_csp += 1;
							total_dc += cacheCpu.get(fedVm);
							vm_dc += 1;

							if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_GOLD) {
								totalCpuGold += cacheCpu.get(fedVm);
								vmCpuGold += 1;
								totalCpuGold_csp += cacheCpu.get(fedVm);
								vmCpuGold_csp += 1;
								totalGold_dc += cacheCpu.get(fedVm);
								vmGold_dc += 1;
							} else if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_SILVER) {
								totalCpuSilver += cacheCpu.get(fedVm);
								vmCpuSilver += 1;
								totalCpuSilver_csp += cacheCpu.get(fedVm);
								vmCpuSilver_csp += 1;
								totalSilver_dc += cacheCpu.get(fedVm);
								vmSilver_dc += 1;
							} else if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_BRONZE) {
								totalCpuBronze += cacheCpu.get(fedVm);
								vmCpuBronze += 1;
								totalCpuBronze_csp += cacheCpu.get(fedVm);
								vmCpuBronze_csp += 1;
								totalBronze_dc += cacheCpu.get(fedVm);
								vmBronze_dc += 1;
							}
						}

					} else {
						for (FederatedVm fedVm : cacheCpu.keySet()) {

							totalCpu += cacheCpu.get(fedVm);
							vmCpu += 1;
							total_dc += cacheCpu.get(fedVm);
							vm_dc += 1;

							if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_GOLD) {
								totalCpuGold += cacheCpu.get(fedVm);
								vmCpuGold += 1;
								totalGold_dc += cacheCpu.get(fedVm);
								vmGold_dc += 1;
							} else if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_SILVER) {
								totalCpuSilver += cacheCpu.get(fedVm);
								vmCpuSilver += 1;
								totalSilver_dc += cacheCpu.get(fedVm);
								vmSilver_dc += 1;
							} else if (fedVm.getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_BRONZE) {
								totalCpuBronze += cacheCpu.get(fedVm);
								vmCpuBronze += 1;
								totalBronze_dc += cacheCpu.get(fedVm);
								vmBronze_dc += 1;
							}
						}
					}

					dcBrokenCpuString.put(fedDc, total_dc + "/" + vm_dc + "," + totalGold_dc + "/" + vmGold_dc + ","
							+ totalSilver_dc + "/" + vmSilver_dc + "," + totalBronze_dc + "/" + vmBronze_dc);
				}

			}

			cspBrokenRamString.put(csp,
					totalRam_csp + "/" + vmRam_csp + "," + totalRamGold_csp + "/" + vmRamGold_csp + ","
							+ totalRamSilver_csp + "/" + vmRamSilver_csp + "," + totalRamBronze_csp + "/"
							+ vmRamBronze_csp);
			cspBrokenCpuString.put(csp,
					totalCpu_csp + "/" + vmCpu_csp + "," + totalCpuGold_csp + "/" + vmCpuGold_csp + ","
							+ totalCpuSilver_csp + "/" + vmCpuSilver_csp + "," + totalCpuBronze_csp + "/"
							+ vmCpuBronze_csp);

			// ExcelLogging
			this.cspTotalBrokenCpu.put(csp, totalCpu_csp);
			this.cspTotalBrokenRam.put(csp, totalRam_csp);

		}

		federationBrokenRamString = totalRam + "/" + vmRam + "," + totalRamGold + "/" + vmRamGold + "," + totalRamSilver
				+ "/" + vmRamSilver + "," + totalRamBronze + "/" + vmRamBronze;
		federationBrokenCpuString = totalCpu + "/" + vmCpu + "," + totalCpuGold + "/" + vmCpuGold + "," + totalCpuSilver
				+ "/" + vmCpuSilver + "," + totalCpuBronze + "/" + vmCpuBronze;

	}

	/*
	 * 
	 * ***** SLA WARNINGS *****
	 * 
	 */

	// ADD Warnings

	public String getFederationBrokenRamString() {
		return federationBrokenRamString;
	}

	public String getFederationBrokenCpuString() {
		return federationBrokenCpuString;
	}

	protected String getCspBrokenRamString(CloudServiceProvider csp) {
		String ret = "0/0,0/0,0/0,0/0";

		if (cspBrokenRamString.containsKey(csp))
			ret = cspBrokenRamString.get(csp);

		return ret;
	}

	protected String getCspBrokenCpuString(CloudServiceProvider csp) {
		String ret = "0/0,0/0,0/0,0/0";

		if (cspBrokenCpuString.containsKey(csp))
			ret = cspBrokenCpuString.get(csp);

		return ret;
	}

	protected String getDcBrokenRamString(FederatedDatacenter fedDc) {
		String ret = "0/0,0/0,0/0,0/0";

		if (dcBrokenRamString.containsKey(fedDc))
			ret = dcBrokenRamString.get(fedDc);

		return ret;
	}

	protected String getDcBrokenCpuString(FederatedDatacenter fedDc) {
		String ret = "0/0,0/0,0/0,0/0";

		if (dcBrokenCpuString.containsKey(fedDc))
			ret = dcBrokenCpuString.get(fedDc);

		return ret;
	}

	public synchronized void addSlaWarningCallsCount_csp(FederatedDatacenter fedDc) {
		if (slaWarningsCount_csp.containsKey(fedDc))
			slaWarningsCount_csp.put(fedDc, slaWarningsCount_csp.get(fedDc) + 1);
		else
			slaWarningsCount_csp.put(fedDc, 1);
	}

	public void addRamThWarningCount_csp(FederatedDatacenter fedDc) {
		if (ramThWarningCount_csp.containsKey(fedDc))
			ramThWarningCount_csp.put(fedDc, ramThWarningCount_csp.get(fedDc) + 1);
		else
			ramThWarningCount_csp.put(fedDc, 1);
	}

	public void addRamDangerThWarningCount_csp(FederatedDatacenter fedDc) {
		if (ramDangerThWarningCount_csp.containsKey(fedDc))
			ramDangerThWarningCount_csp.put(fedDc, ramDangerThWarningCount_csp.get(fedDc) + 1);
		else
			ramDangerThWarningCount_csp.put(fedDc, 1);
	}

	public void addCpuThWarningCount_csp(FederatedDatacenter fedDc) {
		if (cpuThWarningCount_csp.containsKey(fedDc))
			cpuThWarningCount_csp.put(fedDc, cpuThWarningCount_csp.get(fedDc) + 1);
		else
			cpuThWarningCount_csp.put(fedDc, 1);
	}

	public void addCpuDangerThWarningCount_csp(FederatedDatacenter fedDc) {
		if (cpuDangerThWarningCount_csp.containsKey(fedDc))
			cpuDangerThWarningCount_csp.put(fedDc, cpuDangerThWarningCount_csp.get(fedDc) + 1);
		else
			cpuDangerThWarningCount_csp.put(fedDc, 1);
	}

	public synchronized void addConsolidationCall_csp(FederatedDatacenter fedDc) {
		if (consolidationCallCount_csp.containsKey(fedDc))
			consolidationCallCount_csp.put(fedDc, consolidationCallCount_csp.get(fedDc) + 1);
		else
			consolidationCallCount_csp.put(fedDc, 1l);
	}

	// GET Warnings

	public int getSlaWarningCallsCount_csp() {
		int ret = 0;

		for (FederatedDatacenter fedDc : slaWarningsCount_csp.keySet())
			ret += slaWarningsCount_csp.get(fedDc);

		return ret;
	}

	public int getRamThWarningCount_csp() {
		int ret = 0;

		for (FederatedDatacenter fedDc : ramThWarningCount_csp.keySet())
			ret += ramThWarningCount_csp.get(fedDc);

		return ret;
	}

	public int getRamDangerThWarningCount_csp() {
		int ret = 0;

		for (FederatedDatacenter fedDc : ramDangerThWarningCount_csp.keySet())
			ret += ramDangerThWarningCount_csp.get(fedDc);

		return ret;
	}

	public int getCpuThWarningCount_csp() {
		int ret = 0;

		for (FederatedDatacenter fedDc : cpuThWarningCount_csp.keySet())
			ret += cpuThWarningCount_csp.get(fedDc);

		return ret;
	}

	public int getCpuDangerThWarningCount_csp() {
		int ret = 0;

		for (FederatedDatacenter fedDc : cpuDangerThWarningCount_csp.keySet())
			ret += cpuDangerThWarningCount_csp.get(fedDc);

		return ret;
	}

	public long getConsolidationCallCount_csp() {
		int ret = 0;

		for (FederatedDatacenter fedDc : consolidationCallCount_csp.keySet())
			ret += consolidationCallCount_csp.get(fedDc);

		return ret;
	}

	protected int getSlaWarningCallsCount_csp(FederatedDatacenter fedDc) {
		int ret = 0;

		if (slaWarningsCount_csp.containsKey(fedDc))
			ret = slaWarningsCount_csp.get(fedDc);

		return ret;
	}

	protected int getRamThWarningCount_csp(FederatedDatacenter fedDc) {
		int ret = 0;

		if (ramThWarningCount_csp.containsKey(fedDc))
			ret = ramThWarningCount_csp.get(fedDc);

		return ret;
	}

	protected int getRamDangerThWarningCount_csp(FederatedDatacenter fedDc) {
		int ret = 0;

		if (ramDangerThWarningCount_csp.containsKey(fedDc))
			ret = ramDangerThWarningCount_csp.get(fedDc);

		return ret;
	}

	protected int getCpuThWarningCount_csp(FederatedDatacenter fedDc) {
		int ret = 0;

		if (cpuThWarningCount_csp.containsKey(fedDc))
			ret = cpuThWarningCount_csp.get(fedDc);

		return ret;
	}

	protected int getCpuDangerThWarningCount_csp(FederatedDatacenter fedDc) {
		int ret = 0;

		if (cpuDangerThWarningCount_csp.containsKey(fedDc))
			ret = cpuDangerThWarningCount_csp.get(fedDc);

		return ret;
	}

	protected long getConsolidationCallCount_csp(FederatedDatacenter fedDc) {
		long ret = 0;

		if (consolidationCallCount_csp.containsKey(fedDc))
			ret = consolidationCallCount_csp.get(fedDc);

		return ret;
	}

	/*
	 ***** Mean-Values *****
	 */

	/*
	 ***** Migration Counts *****
	 */

	public void addIntraDcMigration(FederatedDatacenter newFedDc, FederatedVm fedVm) {
		if (intraDcMigrationCount.get(newFedDc) == null)
			intraDcMigrationCount.put(newFedDc, 1l);
		else
			intraDcMigrationCount.put(newFedDc, intraDcMigrationCount.get(newFedDc) + 1);
	}

	public void addIntraCspMigration(CloudServiceProvider newCsp, FederatedVm fedVm) {
		if (intraCspMigrationCount.get(newCsp) == null)
			intraCspMigrationCount.put(newCsp, 1l);
		else
			intraCspMigrationCount.put(newCsp, intraCspMigrationCount.get(newCsp) + 1);
	}

	/**
	 * Adds the intra-federation migration to the Endlog. Must be called after
	 * the migration! (called in
	 * {@code FederatedVmAllocationPolicy.allocateHostForVm(FederatedVm)} and
	 * {@code FederatedVmAllocationPolicy.allocateHostForVm(FederatedVm, FederatedHost)})
	 * 
	 * 
	 * @param fedVm
	 *            The FederatedVm object which has been migrated to another CSP
	 */
	public void addIntraFedMigration(CloudServiceProvider oldCsp, CloudServiceProvider newCsp, FederatedVm fedVm) {
		intraFedMigCount += 1;

		/*
		 * Add outgoing migration
		 */

		// Own vm is leaving
		if (oldCsp.getOwnVms().contains(fedVm)) {
			Long oldValue = null;

			if ((oldValue = outgoingMigrationCountCsp_ownVm.get(oldCsp)) != null)
				outgoingMigrationCountCsp_ownVm.put(oldCsp, oldValue + 1);
			else
				outgoingMigrationCountCsp_ownVm.put(oldCsp, 1l);
		}

		// Foreign vm is leaving
		else {
			Long oldValue = null;

			if ((oldValue = outgoingMigrationCountCsp_foreignVm.get(oldCsp)) != null)
				outgoingMigrationCountCsp_foreignVm.put(oldCsp, oldValue + 1);

			else
				outgoingMigrationCountCsp_foreignVm.put(oldCsp, 1l);
		}

		/*
		 * Add incoming migration
		 */

		// Own vm is coming
		if (newCsp.getOwnVms().contains(fedVm)) {
			Long oldValue = null;

			if ((oldValue = incomingMigrationCountCsp_ownVm.get(newCsp)) != null)
				incomingMigrationCountCsp_ownVm.put(newCsp, oldValue + 1);
			else
				incomingMigrationCountCsp_ownVm.put(newCsp, 1l);
		}

		// Foreign vm is coming
		else {
			Long oldValue = null;
			if ((oldValue = incomingMigrationCountCsp_foreignVm.get(newCsp)) != null)
				incomingMigrationCountCsp_foreignVm.put(newCsp, oldValue + 1);
			else
				incomingMigrationCountCsp_foreignVm.put(newCsp, 1l);
		}
	}

	public Long getIntraDcMigrationCountFederationSum() {
		long ret = 0;

		for (FederatedDatacenter fedDc : intraDcMigrationCount.keySet())
			ret += intraDcMigrationCount.get(fedDc);

		return ret;
	}

	public Long getIntraCspMigrationCountFederationSum() {
		long ret = 0;

		for (CloudServiceProvider csp : intraCspMigrationCount.keySet())
			ret += intraCspMigrationCount.get(csp);

		return ret;
	}

	public Long getIntraFedMigrationCountFederationSum() {
		return intraFedMigCount;
	}

	protected long getIntraDcMigrationCount(FederatedDatacenter fedDc) {
		Long ret = -1l;
		if ((ret = intraDcMigrationCount.get(fedDc)) == null)
			ret = 0l;

		return ret;
	}

	protected long getIntraCspMigrationCount(CloudServiceProvider csp) {
		Long ret = -1l;
		if ((ret = intraCspMigrationCount.get(csp)) == null)
			ret = 0l;

		return ret;
	}

	protected long getIncomingMigrationCount(CloudServiceProvider csp) {
		Long ret = 0l;
		Long own = null;
		Long foreign = null;

		if ((own = incomingMigrationCountCsp_ownVm.get(csp)) != null)
			ret += own;
		if (((foreign = incomingMigrationCountCsp_foreignVm.get(csp)) != null))
			ret += foreign;

		return ret;
	}

	protected Long getOutgoingMigrationCount(CloudServiceProvider csp) {
		Long ret = 0l;
		Long own = null;
		Long foreign = null;

		if ((own = outgoingMigrationCountCsp_ownVm.get(csp)) != null)
			ret += own;
		if (((foreign = outgoingMigrationCountCsp_foreignVm.get(csp)) != null))
			ret += foreign;

		return ret;
	}

	/*
	 ***** Other *****
	 */

	@Override
	public void startEntity() {
		Log.printLine(getName() + " is starting...");
		// this resource should register to regional GIS.
		// However, if not specified, then register to system GIS (the
		// default CloudInformationService) entity.
		int gisID = CloudSim.getEntityId(regionalCisName);
		if (gisID == -1) {
			gisID = CloudSim.getCloudInfoServiceEntityId();
		}

		// send the registration to GIS
		sendNow(gisID, CloudSimTags.REGISTER_RESOURCE, getId());
		// Below method is for a child class to override
		// registerOtherEntity();
	}

	@Override
	public void processEvent(SimEvent ev) {
		switch (ev.getTag()) {
		case FederatedCloudSimTags.ENDLOG_TIC_BASED_CALCULATION:
			processTicBasedCalculation();
			break;
		case FederatedCloudSimTags.ENDLOG_SIM_START:
			processSimStart();
			break;
		case FederatedCloudSimTags.ENDLOG_SIM_END:
			processSimEnd();
			break;
		default:
			break;
		}
	}

	private void processSimEnd() {
		simEnd = System.currentTimeMillis();
	}

	private void processSimStart() {
		simStart = System.currentTimeMillis();
	}

	private void processTicBasedCalculation() {

		// mean value calculation
		if (ofPrevention)
			processMeanValuesOFP();
		else
			processMeanValues();

		processMaxValues();

		if (energyLogOn)
			processEnergyLogging();

	}

	private void processMaxValues() {
		for (CloudServiceProvider csp : ServiceProviderRegistry.getInstance().getCspList()) {
			int cspRam = 0;
			double cspCpu = 0.0;

			for (FederatedDatacenter fedDc : csp.getDcList()) {

				if (!maxRamDC.containsKey(fedDc))
					maxRamDC.put(fedDc, 0);
				if (!maxCpuDC.containsKey(fedDc))
					maxCpuDC.put(fedDc, 0.0);

				if (!maxRamCSP.containsKey(csp))
					maxRamCSP.put(csp, 0);
				if (!maxCpuCSP.containsKey(csp))
					maxCpuCSP.put(csp, 0.0);

				int ram = 0;// fedDc.getResources().getRam().getUsed() / 1024;
				double cpu = 0.0;// fedDc.getResources().getMips().getUsed() /
									// 1024;

				for (Host calcHost : fedDc.getHostList()) {
					FederatedHost fedCalcHost = (FederatedHost) calcHost;

					if (fedCalcHost.getResources().getRam().getUsed()
							/ 1024 > fedCalcHost.getResources().getRam().getTotal() / 1024)
						ram += fedCalcHost.getResources().getRam().getTotal() / 1024;
					else
						ram += fedCalcHost.getResources().getRam().getUsed() / 1024;

					if (fedCalcHost.getResources().getMips().getUsed()
							/ 1024 > fedCalcHost.getResources().getMips().getTotal() / 1024)
						cpu += fedCalcHost.getResources().getMips().getTotal() / 1024;
					else
						cpu += fedCalcHost.getResources().getMips().getUsed() / 1024;
				}

				// double debugHostMipsUsed = 0.0;
				// for (Host debugHost : fedDc.getHostList())
				// debugHostMipsUsed += ((FederatedHost)
				// debugHost).getResources().getMips().getUsed() / 1024;

				cspRam += ram;
				cspCpu += cpu;

				if (ram > maxRamDC.get(fedDc))
					maxRamDC.put(fedDc, ram);
				if (cpu > maxCpuDC.get(fedDc))
					maxCpuDC.put(fedDc, cpu);

				// double debug = fedDc.getResources().getMips().getTotal() /
				// 1024;

			}

			if (cspRam > maxRamCSP.get(csp))
				maxRamCSP.put(csp, cspRam);
			if (cspCpu > maxCpuCSP.get(csp))
				maxCpuCSP.put(csp, cspCpu);
		}
	}

	protected String getMaxRamString(CloudServiceProvider csp) {
		String ret = "";

		for (FederatedDatacenter fedDc : csp.getDcList())
			ret += "DC-" + fedDc.getId() + ": " + maxRamDC.get(fedDc) + ", ";

		ret = "CSP: " + maxRamCSP.get(csp) + ", " + ret;

		// removing last ", "
		if (ret.length() >= 1)
			ret = ret.substring(0, ret.length() - 2);

		return ret;
	}

	protected String getMaxCpuString(CloudServiceProvider csp) {
		String ret = "";

		for (FederatedDatacenter fedDc : csp.getDcList())
			ret += "DC-" + fedDc.getId() + ": " + nf.format(maxCpuDC.get(fedDc)) + ", ";

		ret = "CSP: " + nf.format(maxCpuCSP.get(csp)) + ", " + ret;

		// removing last ", "
		if (ret.length() >= 1)
			ret = ret.substring(0, ret.length() - 2);

		return ret;
	}

	protected String getMaxRamStringPercentage(CloudServiceProvider csp) {
		String ret = "";
		long cspTotalRam = 0;

		for (FederatedDatacenter fedDc : csp.getDcList()) {
			int dcTotalRam = fedDc.getResources().getRam().getTotal() / 1024;
			cspTotalRam += dcTotalRam;
			ret += "DC-" + fedDc.getId() + ": "
					+ nf.format(Math.round(((double) (maxRamDC.get(fedDc) * 100)) / dcTotalRam)) + ", ";
		}

		ret = "CSP: " + nf.format(Math.round(((double) (maxRamCSP.get(csp) * 100)) / cspTotalRam)) + ", " + ret;

		// removing last ", "
		if (ret.length() >= 1)
			ret = ret.substring(0, ret.length() - 2);

		return ret;
	}

	protected String getMaxCpuStringPercentage(CloudServiceProvider csp) {
		String ret = "";
		long cspTotalCpu = 0;

		for (FederatedDatacenter fedDc : csp.getDcList()) {
			double dcTotalCpu = fedDc.getResources().getMips().getTotal() / 1024;
			cspTotalCpu += dcTotalCpu;
			ret += "DC-" + fedDc.getId() + ": "
					+ nf.format(Math.round(((double) (maxCpuDC.get(fedDc) * 100)) / dcTotalCpu)) + ", ";
		}

		ret = "CSP: " + nf.format((Math.round(((double) (maxCpuCSP.get(csp) * 100)) / cspTotalCpu))) + ", " + ret;

		// removing last ", "
		if (ret.length() >= 1)
			ret = ret.substring(0, ret.length() - 2);

		return ret;
	}

	private void processMeanValuesOFP() {
		for (CloudServiceProvider csp : ServiceProviderRegistry.getInstance().getCspList()) {

			double meanValCpu_CSP = csp.getResources().getMips().getUsed();
			int meanValRam_CSP = csp.getResources().getRam().getUsed();

			if (meanValuesCpuAct_CSP.containsKey(csp) && meanValuesRamAct_CSP.containsKey(csp)) {

				/*
				 * CPU-CSP
				 */
				if ((Double.MAX_VALUE - meanValuesCpuAct_CSP.get(csp)) <= +meanValCpu_CSP) {

					meanValuesCpu_CSP.get(csp).add(meanValuesCpuAct_CSP.get(csp) / meanActCountCpu_CSP);
					mean_Count_Cpu_CSP.get(csp).add(meanActCountCpu_CSP);
					meanValuesCpuAct_CSP.put(csp, 0.0);
					meanActCountCpu_CSP = 0;

				} else {
					meanValuesCpuAct_CSP.put(csp, meanValuesCpuAct_CSP.get(csp) + meanValCpu_CSP);
					meanActCountCpu_CSP++;
				}

				/*
				 * RAM-CSP
				 */
				if ((Integer.MAX_VALUE - meanValuesRamAct_CSP.get(csp)) <= meanValRam_CSP) {

					meanValuesRam_CSP.get(csp)
							.add((int) Math.round(meanValuesRamAct_CSP.get(csp) / (double) meanActCountRam_CSP));
					mean_Count_Ram_CSP.get(csp).add(meanActCountRam_CSP);
					meanValuesRamAct_CSP.put(csp, 0);
					meanActCountRam_CSP = 0;

				} else {
					meanValuesRamAct_CSP.put(csp, meanValuesRamAct_CSP.get(csp) + meanValRam_CSP);
					meanActCountRam_CSP++;
				}

			} else {
				meanValuesCpuAct_CSP.put(csp, meanValCpu_CSP);
				meanValuesRamAct_CSP.put(csp, meanValRam_CSP);
			}

			for (FederatedDatacenter fedDc : csp.getDcList()) {

				double meanValCpu_DC = fedDc.getResources().getMips().getUsed();
				int meanValRam_DC = fedDc.getResources().getRam().getUsed();

				if (meanValuesCpuAct_DC.containsKey(fedDc) && meanValuesRamAct_DC.containsKey(fedDc)) {

					/*
					 * CPU-DC
					 */
					if ((Double.MAX_VALUE - meanValuesCpuAct_DC.get(fedDc)) <= +meanValCpu_DC) {

						meanValuesCpu_DC.get(fedDc).add(meanValuesCpuAct_DC.get(fedDc) / meanActCountCpu_DC);
						mean_Count_Cpu_DC.get(fedDc).add(meanActCountCpu_DC);
						meanValuesCpuAct_DC.put(fedDc, 0.0);
						meanActCountCpu_DC = 0;

					} else {
						meanValuesCpuAct_DC.put(fedDc, meanValuesCpuAct_DC.get(fedDc) + meanValCpu_DC);
						meanActCountCpu_DC++;
					}

					/*
					 * RAM-DC
					 */
					if ((Integer.MAX_VALUE - meanValuesRamAct_DC.get(fedDc)) <= meanValRam_DC) {

						meanValuesRam_DC.get(fedDc)
								.add((int) Math.round(meanValuesRamAct_DC.get(fedDc) / (double) meanActCountRam_DC));
						mean_Count_Ram_DC.get(fedDc).add(meanActCountRam_DC);
						meanValuesRamAct_DC.put(fedDc, 0);
						meanActCountRam_DC = 0;

					} else {
						meanValuesRamAct_DC.put(fedDc, meanValuesRamAct_DC.get(fedDc) + meanValRam_DC);
						meanActCountRam_DC++;
					}

				} else {
					meanValuesCpuAct_DC.put(fedDc, meanValCpu_DC);
					meanValuesRamAct_DC.put(fedDc, meanValRam_DC);
				}
			}
		}
	}

	private int debugCount = 0;

	private boolean energyLogOn;

	private void processMeanValues() {
		meanCount++;

		for (CloudServiceProvider csp : ServiceProviderRegistry.getInstance().getCspList())

			for (FederatedDatacenter fedDc : csp.getDcList()) {

				double meanValCpu_CSP = csp.getResources().getMips().getUsed() / 1024;
				int meanValRam_CSP = csp.getResources().getRam().getUsed() / 1024;
				double meanValCpu = fedDc.getResources().getMips().getUsed() / 1024;
				int meanValRam = fedDc.getResources().getRam().getUsed() / 1024;

				if (meanValuesCpuAct_DC.containsKey(fedDc) && meanValuesRamAct_DC.containsKey(fedDc)) {

					// TODO DEBUG
					if ((Double.MAX_VALUE - meanValuesCpuAct_DC.get(fedDc)) <= meanValCpu)
						System.out.println("1 :: DOUBLE-MAX-VALUE REACHED! (while calculating meanLoads)");

					if ((Integer.MAX_VALUE - meanValuesRamAct_DC.get(fedDc)) <= meanValRam)
						System.out.println("2 :: DOUBLE-MAX-VALUE REACHED! (while calculating meanLoads) :: "
								+ debugCount + " :: " + meanValuesRamAct_DC.get(fedDc) + " :: " + Integer.MAX_VALUE);
					else
						debugCount++;

					meanValuesCpuAct_DC.put(fedDc, meanValuesCpuAct_DC.get(fedDc) + meanValCpu);
					meanValuesRamAct_DC.put(fedDc, meanValuesRamAct_DC.get(fedDc) + meanValRam);

				} else {
					meanValuesCpuAct_DC.put(fedDc, meanValCpu);
					meanValuesRamAct_DC.put(fedDc, meanValRam);
				}

				if (meanValuesCpuAct_CSP.containsKey(csp) && meanValuesRamAct_CSP.containsKey(csp)) {

					// TODO DEBUG
					if ((Double.MAX_VALUE - meanValuesCpuAct_CSP.get(csp)) <= meanValCpu_CSP)
						System.out.println("3 :: CSP - DOUBLE-MAX-VALUE REACHED! (while calculating meanLoads)");

					if ((Integer.MAX_VALUE - meanValuesRamAct_CSP.get(csp)) <= meanValRam_CSP)
						System.out.println("4 :: CSP - DOUBLE-MAX-VALUE REACHED! (while calculating meanLoads)");

					meanValuesCpuAct_CSP.put(csp, meanValuesCpuAct_CSP.get(csp) + meanValCpu_CSP);
					meanValuesRamAct_CSP.put(csp, meanValuesRamAct_CSP.get(csp) + meanValRam_CSP);

				} else {
					meanValuesCpuAct_CSP.put(csp, meanValCpu_CSP);
					meanValuesRamAct_CSP.put(csp, meanValRam_CSP);
				}

			}
	}

	protected void calculateMeanValues() {

		if (!ofPrevention)
			return;

		for (CloudServiceProvider csp : ServiceProviderRegistry.getInstance().getCspList()) {

			if (meanValuesCpu_CSP.containsKey(csp) && meanValuesRam_CSP.containsKey(csp)) {

				meanValuesCpu_CSP.get(csp).add(meanValuesCpuAct_CSP.get(csp) / meanActCountCpu_CSP);
				mean_Count_Cpu_CSP.get(csp).add(meanActCountCpu_CSP);

				meanValuesRam_CSP.get(csp)
						.add((int) Math.round(meanValuesRamAct_CSP.get(csp) / (double) meanActCountRam_CSP));
				mean_Count_Ram_CSP.get(csp).add(meanActCountRam_CSP);

				meanActCountCpu_CSP = 0;
				meanActCountRam_CSP = 0;
				meanValuesCpuAct_CSP.put(csp, 0.0);
				meanValuesRamAct_CSP.put(csp, 0);

			}

			for (FederatedDatacenter fedDc : csp.getDcList()) {

				if (meanValuesCpu_DC.containsKey(fedDc) && meanValuesRam_DC.containsKey(fedDc)) {

					meanValuesCpu_DC.get(fedDc).add(meanValuesCpuAct_DC.get(fedDc) / meanActCountCpu_DC);
					mean_Count_Cpu_DC.get(fedDc).add(meanActCountCpu_DC);

					meanValuesRam_DC.get(fedDc)
							.add((int) Math.round(meanValuesRamAct_DC.get(fedDc) / (double) meanActCountRam_DC));
					mean_Count_Ram_DC.get(fedDc).add(meanActCountRam_DC);

					meanActCountCpu_DC = 0;
					meanActCountRam_DC = 0;
					meanValuesCpuAct_DC.put(fedDc, 0.0);
					meanValuesRamAct_DC.put(fedDc, 0);

				}
			}
		}
	}

	private long calcMeanValueRam(FederatedDatacenter fedDc) {
		long ret = 0;

		if (!ofPrevention) {

			ret = meanValuesRamAct_DC.get(fedDc) / meanCount;

		} else {

			List<Integer> values = meanValuesRam_DC.get(fedDc);
			List<Long> counts = mean_Count_Ram_DC.get(fedDc);

			for (int i = 0; i < values.size(); i++) {
				ret += values.get(i) * (counts.get(i) / meanCount);
			}
		}

		return ret;
	}

	private double calcMeanValueCpu(FederatedDatacenter fedDc) {
		double ret = 0;

		if (!ofPrevention) {

			ret = meanValuesCpuAct_DC.get(fedDc) / meanCount;

		} else {

			List<Double> values = meanValuesCpu_DC.get(fedDc);
			List<Long> counts = mean_Count_Cpu_DC.get(fedDc);

			for (int i = 0; i < values.size(); i++) {
				ret += values.get(i) * (counts.get(i) / meanCount);
			}
		}

		return ret;
	}

	private long calcMeanValueRam(CloudServiceProvider csp) {
		long ret = 0;

		List<Integer> values = meanValuesRam_CSP.get(csp);
		List<Long> counts = mean_Count_Ram_CSP.get(csp);

		for (int i = 0; i < values.size(); i++) {
			ret += values.get(i) * (counts.get(i) / meanCount);
		}

		return ret;
	}

	private double calcMeanValueCpu(CloudServiceProvider csp) {
		long ret = 0;

		List<Double> values = meanValuesCpu_CSP.get(csp);
		List<Long> counts = mean_Count_Cpu_CSP.get(csp);

		for (int i = 0; i < values.size(); i++) {
			ret += values.get(i) * (counts.get(i) / meanCount);
		}

		return ret;
	}

	protected String getMeanValueRamString(CloudServiceProvider csp) {
		String ret = "";
		long cspTotal = 0;

		for (FederatedDatacenter fedDc : csp.getDcList()) {
			long meanRam = calcMeanValueRam(fedDc);

			if (ofPrevention) {
				cspTotal += Math.round(meanRam / (double) 1024);
				ret += "DC-" + fedDc.getId() + ": " + Math.round(meanRam / (double) 1024) + ", ";
			} else {
				cspTotal += meanRam;
				ret += "DC-" + fedDc.getId() + ": " + meanRam + ", ";
			}
		}

		ret = "CSP: " + cspTotal + ", " + ret;

		// removing last ", "
		if (ret.length() >= 1)
			ret = ret.substring(0, ret.length() - 2);

		return ret;
	}

	protected String getMeanValueCpuString(CloudServiceProvider csp) {
		String ret = "";
		long cspTotal = 0;

		for (FederatedDatacenter fedDc : csp.getDcList()) {
			double meanCpu = calcMeanValueCpu(fedDc);

			if (ofPrevention) {
				cspTotal += Math.round(meanCpu / 1024);
				ret += "DC-" + fedDc.getId() + ": " + Math.round(meanCpu / 1024) + ", ";
			} else {
				cspTotal += meanCpu;
				ret += "DC-" + fedDc.getId() + ": " + Math.round(meanCpu) + ", ";
			}
		}

		ret = "CSP: " + cspTotal + ", " + ret;

		// removing last ", "
		if (ret.length() >= 1)
			ret = ret.substring(0, ret.length() - 2);

		return ret;
	}

	protected List<String> getPrintCacheMeanVal(CloudServiceProvider csp) {
		long meanRam = 0;
		double meanCpu = 0;
		List<String> ret = new ArrayList<String>();

		for (FederatedDatacenter fedDc : csp.getDcList()) {
			meanRam = calcMeanValueRam(fedDc);
			meanCpu = calcMeanValueCpu(fedDc);
			ret.add(nf.format(Math.round((meanRam * 100) / (fedDc.getResources().getRam().getTotal() / (double) 1024))));
			ret.add(nf.format(Math.round((meanCpu * 100) / (fedDc.getResources().getMips().getTotal() / (double) 1024))));
		}
			
		return ret;
	}

	protected String getMeanValueRamStringPercentage(CloudServiceProvider csp) {
		String ret = "";
		double cspTotal = 0;

		for (FederatedDatacenter fedDc : csp.getDcList()) {
			long meanRam = calcMeanValueRam(fedDc);

			if (ofPrevention) {
				cspTotal += (Math.round((meanRam * 100) / (double) fedDc.getResources().getRam().getTotal()));
				ret += "DC-" + fedDc.getId() + ": "
						+ nf.format(Math.round((meanRam * 100) / (double) fedDc.getResources().getRam().getTotal()))
						+ ", ";
			} else {
				cspTotal += (Math.round((meanRam * 100) / (fedDc.getResources().getRam().getTotal() / (double) 1024)));
				ret += "DC-" + fedDc.getId() + ": "
						+ nf.format(Math.round((meanRam * 100) / (fedDc.getResources().getRam().getTotal() / (double) 1024)))
						+ ", ";
			}
		}

		ret = "CSP: " + nf.format(cspTotal) + ", " + ret;

		// removing last ", "
		if (ret.length() >= 1)
			ret = ret.substring(0, ret.length() - 2);

		return ret;
	}

	protected String getMeanValueCpuStringPercentage(CloudServiceProvider csp) {
		String ret = "";
		double cspTotal = 0;

		for (FederatedDatacenter fedDc : csp.getDcList()) {
			double meanCpu = calcMeanValueCpu(fedDc);

			if (ofPrevention) {
				cspTotal += (Math.round((meanCpu * 100) / fedDc.getResources().getMips().getTotal()));
				ret += "DC-" + fedDc.getId() + ": "
						+ nf.format(Math.round((meanCpu * 100) / fedDc.getResources().getMips().getTotal())) + ", ";
			} else {
				cspTotal += (Math.round((meanCpu * 100) / (fedDc.getResources().getMips().getTotal() / 1024)));
				ret += "DC-" + fedDc.getId() + ": "
						+ nf.format(Math.round((meanCpu * 100) / (fedDc.getResources().getMips().getTotal() / 1024)))
						+ ", ";
			}
		}

		ret = "CSP: " + nf.format(cspTotal) + ", " + ret;

		// removing last ", "
		if (ret.length() >= 1)
			ret = ret.substring(0, ret.length() - 2);

		return ret;
	}

	protected String getTotalRamValuesString(CloudServiceProvider csp) {
		String ret = "";
		long cspTotal = 0;

		for (FederatedDatacenter fedDc : csp.getDcList()) {
			cspTotal += fedDc.getResources().getRam().getTotal();
			ret += "DC-" + fedDc.getId() + ": " + fedDc.getResources().getRam().getTotal() / 1024 + ", ";
		}

		ret = "CSP: " + cspTotal / 1024 + ", " + ret;

		// removing last ", "
		if (ret.length() >= 1)
			ret = ret.substring(0, ret.length() - 2);

		return ret;
	}

	protected String getTotalCpuValuesString(CloudServiceProvider csp) {
		String ret = "";
		double cspTotal = 0;

		for (FederatedDatacenter fedDc : csp.getDcList()) {
			cspTotal += fedDc.getResources().getMips().getTotal();
			ret += "DC-" + fedDc.getId() + ": " + Math.round(fedDc.getResources().getMips().getTotal() / 1024) + ", ";
		}

		ret = "CSP: " + Math.round(cspTotal / 1024) + ", " + ret;

		// removing last ", "
		if (ret.length() >= 1)
			ret = ret.substring(0, ret.length() - 2);

		return ret;
	}

	@Override
	public void shutdownEntity() {
		Log.printLine(getName() + " is shutting down...");
	}

	public void setRegionalCisName(String regionalCisName) {
		this.regionalCisName = regionalCisName;
	}

	public String getRegionalCisName() {
		return this.regionalCisName;
	}

	public void setSimConfigName(String name) {
		simConfigName = name;
	}

	public String getSimConfigName() {
		return simConfigName;
	}

	public String getRuntime() {
		long min = ((simEnd - simStart) / 1000) / 60;
		long sec_cache = ((simEnd - simStart) / 1000);
		long sec = sec_cache % 60;

		String ret = "";

		if (sec < 10)
			ret = min + ":0" + sec;
		else
			ret = min + ":" + sec;

		return ret;
	}

	public long getSimStart() {
		return simStart;
	}

	public long getSimEnd() {
		return simEnd;
	}

	public long getHostOnCounter() {
		return hostOnCounter;
	}

	public long getHostTotalCounter() {
		return hostTotalCounter;
	}

	public void addMigrationCountDc(FederatedDatacenter source, FederatedDatacenter destination) {
		Map<FederatedDatacenter, Long> outgoingMap = null;

		if (!outgoingMigrationsCountDc.containsKey(source))
			outgoingMigrationsCountDc.put(source, new HashMap<FederatedDatacenter, Long>());

		outgoingMap = outgoingMigrationsCountDc.get(source);

		if (!outgoingMap.containsKey(destination))
			outgoingMap.put(destination, 1l);
		else
			outgoingMap.put(destination, (outgoingMap.get(destination) + 1));

		Map<FederatedDatacenter, Long> incomingMap = null;

		if (!incomingMigrationsCountDc.containsKey(destination))
			incomingMigrationsCountDc.put(destination, new HashMap<FederatedDatacenter, Long>());

		incomingMap = incomingMigrationsCountDc.get(destination);

		if (!incomingMap.containsKey(source))
			incomingMap.put(source, 1l);
		else
			incomingMap.put(source, (incomingMap.get(source) + 1));
	}

	public Set<FederatedDatacenter> getMigrationCountOutgoingDcKeys() {
		return outgoingMigrationsCountDc.keySet();
	}

	public Set<FederatedDatacenter> getMigrationCountIncomingDcKeys() {
		return incomingMigrationsCountDc.keySet();
	}

	public void addMigrationCountHost(FederatedHost source, FederatedHost destination) {
		Map<FederatedHost, Long> outgoingMap = null;

		if (!outgoingMigrationsCountHost.containsKey(source))
			outgoingMigrationsCountHost.put(source, new HashMap<FederatedHost, Long>());

		outgoingMap = outgoingMigrationsCountHost.get(source);

		if (!outgoingMap.containsKey(destination))
			outgoingMap.put(destination, 1l);
		else
			outgoingMap.put(destination, (outgoingMap.get(destination) + 1));

		Map<FederatedHost, Long> incomingMap = null;

		if (!incomingMigrationsCountDc.containsKey(destination))
			incomingMigrationsCountHost.put(destination, new HashMap<FederatedHost, Long>());

		incomingMap = incomingMigrationsCountHost.get(destination);

		if (!incomingMap.containsKey(source))
			incomingMap.put(source, 1l);
		else
			incomingMap.put(source, (incomingMap.get(source) + 1));
	}

	public Set<FederatedHost> getMigrationCountOutgoingHostKeys() {
		return outgoingMigrationsCountHost.keySet();
	}

	public Set<FederatedHost> getMigrationCountIncomingHostKeys() {
		return incomingMigrationsCountHost.keySet();
	}

	public void setEndtime(int endTime) {
		simEndtime = endTime;
	}

	public int getEndTime() {
		return simEndtime;
	}

	public void setConfigName(String configName) {
		this.configName = configName;
	}

	public String getConfigName() {
		return configName;
	}

	public void setPingPongThreshold(int pingPong) {
		this.pingPong = pingPong;
	}

	public String getPingPongThreshold() {
		return pingPong + "";
	}

	public Map<CloudServiceProvider, Long> getPingPongAvoidanceCountMap() {
		return pingPongAvoidanceCount;
	}

	protected long getPingPongAvoidanceCount(CloudServiceProvider csp) {
		long ret = 0l;

		if (pingPongAvoidanceCount.containsKey(csp))
			ret = pingPongAvoidanceCount.get(csp);

		return ret;
	}

	public void addPingPongCount(CloudServiceProvider csp) {
		if (pingPongAvoidanceCount.containsKey(csp))
			pingPongAvoidanceCount.put(csp, pingPongAvoidanceCount.get(csp) + 1);
		else
			pingPongAvoidanceCount.put(csp, 1l);
	}

	public boolean isEnergyLogOn() {
		// TODO Auto-generated method stub
		return this.energyLogOn;
	}

	public void setEnergyLogOn(boolean value) {
		this.energyLogOn = value;
	}

	public void addEnergyHostOnValue(CloudServiceProvider csp, int hostOnCount) {

	}

	public void processEnergyLogging() {
		for (CloudServiceProvider csp : ServiceProviderRegistry.getInstance().getCspList()) {
			int hostOnCount = 0;

			for (FederatedDatacenter fedDc : csp.getDcList())
				hostOnCount += fedDc.getHostOnList().size();

			hostsOn.get(csp).add(hostOnCount);
		}
	}

	public Map<CloudServiceProvider, List<Integer>> getHostOnMap() {
		return hostsOn;
	}

	public void addHostOnCount() {
		hostOnCount++;
	}

	public void addHostOffCount() {
		hostOffCount++;
	}

	public long getHostOnCount() {
		return hostOnCount;
	}

	public long getHostOffCount() {
		return hostOffCount;
	}

	public void setOcValue(int ocVal) {
		ocValue = ocVal;
	}

	public int getOcValue() {
		return ocValue;
	}

	public int getBrokenCpu(CloudServiceProvider csp) {
		return cspTotalBrokenCpu.get(csp);
	}

	public int getBrokenRam(CloudServiceProvider csp) {
		return cspTotalBrokenRam.get(csp);
	}

}