/**
 * 
 */
package com.materna.federatedcloudsim.monitoring;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.annotation.processing.ProcessingEnvironment;

import org.cloudbus.cloudsim.Host;
import org.cloudbus.cloudsim.Log;
import org.cloudbus.cloudsim.Vm;
import org.cloudbus.cloudsim.core.CloudSimTags;
import org.cloudbus.cloudsim.core.SimEntity;
import org.cloudbus.cloudsim.core.SimEvent;

import com.materna.federatedcloudsim.CloudServiceProvider;
import com.materna.federatedcloudsim.FederatedCloudlet;
import com.materna.federatedcloudsim.FederatedDatacenter;
import com.materna.federatedcloudsim.FederatedHost;
import com.materna.federatedcloudsim.FederatedVm;
import com.materna.federatedcloudsim.core.FederatedCloudSim;
import com.materna.federatedcloudsim.core.FederatedCloudSimTags;
import com.materna.federatedcloudsim.federation.ServiceProviderRegistry;

/**
 * @author Marc Spohr
 * 
 */
public class Monitoring extends SimEntity {

	private boolean logInitalAllocation = false;

	// Workaround for avg Load Monitoring

	private ArrayList<Double> avgCpuDcLoad = new ArrayList<Double>();
	private ArrayList<Double> avgRamDcLoad = new ArrayList<Double>();

	private long tic = 0;
	private long totalTics;

	long percent = 0;
	int check = 0;
	long time = 0;
	long duration = 0;

	public ArrayList<Double> getAvgCpuDcLoad() {
		return avgCpuDcLoad;
	}

	public void setAvgCpuDcLoad(ArrayList<Double> avgCpuDcLoad) {
		this.avgCpuDcLoad = avgCpuDcLoad;
	}

	public ArrayList<Double> getAvgRamDcLoad() {
		return avgRamDcLoad;
	}

	public void setAvgRamDcLoad(ArrayList<Double> avgRamDcLoad) {
		this.avgRamDcLoad = avgRamDcLoad;
	}

	private static final Monitoring instance = new Monitoring();
	private SimEvent monitoringEvent;

	/**
	 * private Constructor to make Monitoring a Singleton
	 */
	private Monitoring() {
		super("FederatedCloudSimMonitoringEntity");
		setMonitoringEvent(new SimEvent());
	}

	/**
	 * @return Monitoring global instance
	 */
	public static Monitoring getInstance() {
		return instance;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.cloudbus.cloudsim.core.SimEntity#startEntity()
	 */
	@Override
	public void startEntity() {
		FileLog.printLine(getName() + " is starting...");
		// trigger initial resource data measurement
		for (CloudServiceProvider csp : ServiceProviderRegistry.getInstance().getCspList())
			for (FederatedDatacenter dc : csp.getDcList())
				for (Host host : dc.getHostList())
					((FederatedHost) host).getResources().update();
		registerOtherEntity();
	}

	/**
	 * Overrides this method when making a new and different type of resource.
	 * <br>
	 * <b>NOTE:</b> You do not need to override {@link #startEntity()} method,
	 * if you use this method.
	 * 
	 * @pre $none
	 * @post $none
	 */
	protected void registerOtherEntity() {
		// empty. This should be override by a child class
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.cloudbus.cloudsim.core.SimEntity#processEvent(org.cloudbus.cloudsim
	 * .core.SimEvent)
	 */
	@Override
	public void processEvent(SimEvent ev) {
		switch (ev.getTag()) {
		case FederatedCloudSimTags.MONITORING:
			processMonitoring(ev);
			break;
		case FederatedCloudSimTags.MONITORING_ENDLOG:
			processEndlog(ev);
			processAuctionLog();
			processEnergyLog();
			break;
		case FederatedCloudSimTags.MONITORING_MIGRATION:
			processMigrationLog(ev);
			break;
		case FederatedCloudSimTags.ACCOUNT_UPDATE:
			processAccountUpdate();
			break;
		case FederatedCloudSimTags.MONITORING_ACCOUNTLOG:
			processAccountLog();
			break;
		case FederatedCloudSimTags.MONITORING_CSV:
			processCsvLog();
			break;
		case FederatedCloudSimTags.MONITORING_TIC:
			processTIC(ev);
			break;
		case FederatedCloudSimTags.MONITORING_GEPHI:
			processGephi();
			break;
		default:
			processOtherEvent(ev);
			break;
		}
	}

	private void processCsvLog() {
		FileLog.getInstance().csv_log();
	}
	
	private void processEnergyLog() {
		if (Endlog.getInstance().isEnergyLogOn())
			FileLog.getInstance().energyLog();
	}

	private void processGephi() {
		try {
			new GephiGenerator(FileLog.getInstance().getBasedir() + File.separator + FileLog.getInstance().getLogdir()
					+ File.separator).generateGephi();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void processTIC(SimEvent ev) {
		this.tic = (Long) ev.getData();

		// status logging (on console)
		this.statusLog();
	}

	public void statusLog() {

		getTotalTics(); // total Tics of Simulation
		// Endlog.getInstance().getSimStart(); // realtime of simulation start
		getTic(); // current tic

		// calculate percentage of completion
		percent = Math.round((tic * 100) / totalTics);

		// calculate simulation runtime
		time = System.currentTimeMillis() - Endlog.getInstance().getSimStart();
		duration = time;

		if (percent == 5 && check == 0) {

			time = ((time * 100) / percent) - duration;
			System.out.println("|>-------------------| Simulation progress: 5%, runtime ~"
					+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
					+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			check = 5;
		}

		if (percent == 10 && check == 5) {
			time = ((time * 100) / percent) - duration;
			System.out.println("|>>------------------| Simulation progress: 10%, runtime ~"
					+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
					+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			check = 10;
		}

		if (percent == 15 && check == 10) {
			time = ((time * 100) / percent) - duration;
			System.out.println("|>>>-----------------| Simulation progress: 15%, runtime ~"
					+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
					+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			check = 15;
		}

		if (percent == 20 && check == 15) {
			time = ((time * 100) / percent) - duration;
			System.out.println("|>>>>----------------| Simulation progress: 20%, runtime ~"
					+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
					+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			check = 20;
		}

		if (percent == 25 && check == 20) {
			time = ((time * 100) / percent) - duration;
			System.out.println("|>>>>>---------------| Simulation progress: 25%, runtime ~"
					+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
					+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			check = 25;
		}

		if (percent == 30 && check == 25) {
			time = ((time * 100) / percent) - duration;
			System.out.println("|>>>>>>--------------| Simulation progress: 30%, runtime ~"
					+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
					+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			check = 30;
		}

		if (percent == 35 && check == 30) {
			time = ((time * 100) / percent) - duration;
			System.out.println("|>>>>>>>-------------| Simulation progress: 35%, runtime ~"
					+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
					+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			check = 35;
		}

		if (percent == 40 && check == 35) {
			time = ((time * 100) / percent) - duration;
			System.out.println("|>>>>>>>>------------| Simulation progress: 40%, runtime ~"
					+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
					+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			check = 40;
		}

		if (percent == 45 && check == 40) {
			time = ((time * 100) / percent) - duration;
			System.out.println("|>>>>>>>>>-----------| Simulation progress: 45%, runtime ~"
					+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
					+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			check = 45;
		}

		if (percent == 50 && check == 45) {
			time = ((time * 100) / percent) - duration;
			System.out.println("|>>>>>>>>>>----------| Simulation progress: 50%, runtime ~"
					+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
					+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			check = 50;
		}

		if (percent == 55 && check == 50) {
			time = ((time * 100) / percent) - duration;
			System.out.println("|>>>>>>>>>>>---------| Simulation progress: 55%, runtime ~"
					+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
					+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			check = 55;
		}

		if (percent == 60 && check == 55) {
			time = ((time * 100) / percent) - duration;
			System.out.println("|>>>>>>>>>>>>--------| Simulation progress: 60%, runtime ~"
					+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
					+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			check = 60;
		}

		if (percent == 65 && check == 60) {
			time = ((time * 100) / percent) - duration;
			System.out.println("|>>>>>>>>>>>>>-------| Simulation progress: 65%, runtime ~"
					+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
					+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			check = 65;
		}

		if (percent == 70 && check == 65) {
			time = ((time * 100) / percent) - duration;
			System.out.println("|>>>>>>>>>>>>>>------| Simulation progress: 70%, runtime ~"
					+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
					+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			check = 70;
		}

		if (percent == 75 && check == 70) {
			time = ((time * 100) / percent) - duration;
			System.out.println("|>>>>>>>>>>>>>>>-----| Simulation progress: 75%, runtime ~"
					+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
					+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			check = 75;
		}

		if (percent == 80 && check == 75) {
			time = ((time * 100) / percent) - duration;
			if ((time / (1000 * 60) % 60) < 1) {
				System.out.println("|>>>>>>>>>>>>>>>>----| Simulation progress: 80%, runtime ~"
						+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in <1 minute\r");
				check = 80;
			} else {
				System.out.println("|>>>>>>>>>>>>>>>>----| Simulation progress: 80%, runtime ~"
						+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
						+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
				check = 80;
			}
		}

		if (percent == 85 && check == 80) {
			time = ((time * 100) / percent) - duration;
			if ((time / (1000 * 60) % 60) < 1) {
				System.out.println("|>>>>>>>>>>>>>>>>>---| Simulation progress: 85%, runtime ~"
						+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in <1 minute\r");
			} else {
				System.out.println("|>>>>>>>>>>>>>>>>>---| Simulation progress: 85%, runtime ~"
						+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated  end in ~"
						+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			}
			check = 85;
		}

		if (percent == 90 && check == 85) {
			time = ((time * 100) / percent) - duration;
			if ((time / (1000 * 60) % 60) < 1) {
				System.out.println("|>>>>>>>>>>>>>>>>>>--| Simulation progress: 90%, runtime ~"
						+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in <1 minute\r");
				check = 90;
			} else {
				System.out.println("|>>>>>>>>>>>>>>>>>>--| Simulation progress: 90%, runtime ~"
						+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
						+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			}

			check = 90;
		}

		if (percent == 95 && check == 90) {
			time = ((time * 100) / percent) - duration;
			if ((time / (1000 * 60) % 60) < 1) {
				System.out.println("|>>>>>>>>>>>>>>>>>>>-| Simulation progress: 95%, runtime ~"
						+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in <1 minute\r");
			} else {
				System.out.println("|>>>>>>>>>>>>>>>>>>>-| Simulation progress: 95%, runtime ~"
						+ ((int) ((duration / (1000 * 60)) % 60)) + " minutes, estimated end in ~"
						+ ((int) ((time / (1000 * 60)) % 60)) + " minutes\r");
			}
			check = 95;
		}

		if (percent == 100 && check == 95) {
			System.out.println("|>>>>>>>>>>>>>>>>>>>>| Simulation progress: 100%, runtime ~"
					+ ((int) ((duration / (1000 * 60)) % 60)));
		}

	}

	private void processMigrationLog(SimEvent ev) {
		FederatedVm vm = (FederatedVm) ev.getData();

		FederatedHost oldHost = null;
		FederatedHost newHost = vm.getFedHostMigHistory().get(vm.getFedHostMigHistory().size() - 1);

		if (vm.getFedHostMigHistory().size() >= 2)
			oldHost = vm.getFedHostMigHistory().get(vm.getFedHostMigHistory().size() - 2);

		String slaLevel = "";
		Set<FederatedCloudlet> fedCls = vm.getCloudlets();

		Iterator<FederatedCloudlet> it = fedCls.iterator();
		while (it.hasNext()) {
			slaLevel += it.next().getSr().getSla().getLevel();

			if (it.hasNext())
				slaLevel += ", ";
		}

		FileLog.getInstance().migration_log(oldHost, newHost, vm.getId(), slaLevel, vm, ev);
	}

	/**
	 * Updates the Accounting for every CSP
	 */
	private void processAccountUpdate() {
		for (CloudServiceProvider csp : ServiceProviderRegistry.getInstance().getCspList()) {
			csp.getAccounting().update();
		}
	}

	/**
	 * Create AccountLog using {@link FileLog}
	 */
	private void processAccountLog() {
		FileLog.getInstance().account_log();
	}

	/**
	 * Create AuctionLog using {@link FileLog}
	 */
	private void processAuctionLog() {
		FileLog.getInstance().auction_log();
	}

	/**
	 */
	private void processMonitoring(SimEvent ev) {

		// System.out.println(FederatedCloudSim.clock());

		// update every single vm before logging
		for (CloudServiceProvider csp : ServiceProviderRegistry.getInstance().getCspList()) {
			for (FederatedDatacenter fdc : csp.getDcList()) {

				double dcCpuLoad = 0;
				double dcRamLoad = 0;

				getAvgCpuDcLoad().add(dcCpuLoad);
				getAvgRamDcLoad().add(dcRamLoad);
				for (Host fhst : fdc.getHostList()) {
					fhst.getRamProvisioner().deallocateRamForAllVms();

					for (Vm fvm : fhst.getVmList()) {
						fhst.getRamProvisioner().allocateRamForVm(fvm, fvm.getCurrentRequestedRam());
						((FederatedVm) fvm).getResources().update();

					}
					((FederatedHost) fhst).getResources().update();

					dcRamLoad += ((FederatedHost) fhst).getResources().getRam().getUsed();
					dcCpuLoad += ((FederatedHost) fhst).getResources().getMips().getUsed();

				}

				// dcCpuLoad = dcCpuLoad /
				// fdc.getResources().getMips().getTotal();
				// dcRamLoad = dcRamLoad /
				// fdc.getResources().getRam().getTotal();
				getAvgCpuDcLoad().add(dcCpuLoad);
				getAvgRamDcLoad().add(dcRamLoad);
			}

			// TODO Damian: methode wieder reinkopieren

			// FileLog.getInstance().csp_stats_host(csp);
		}
		if (ev.getData() != null) {

			@SuppressWarnings("unchecked")
			List<SimEvent> eventList = (LinkedList<SimEvent>) ev.getData();
			// inspect events that triggered this monitoring run
			for (SimEvent triggerEvent : eventList) {
				if (triggerEvent.getData() instanceof FederatedCloudlet) {
					// trigger data measurement of dependent entites
					((FederatedCloudlet) triggerEvent.getData()).getVm().getResources().update();
				}
			}
			// if(triggerLog)
			// write logfile
			for (CloudServiceProvider csp : ServiceProviderRegistry.getInstance().getCspList())
				FileLog.getInstance().csp_stats_host(csp);
		}
	}

	private void processEndlog(SimEvent ev) {
		FileLog.getInstance().endlog_vm_summary();
	}

	/**
	 * Overrides this method when making a new and different type of Broker.
	 * This method is called by {@link #processEvent(SimEvent)} for incoming
	 * unknown tags.
	 * 
	 * @param ev
	 *            the {@link SimEvent} object to be processed
	 * @pre ev != null
	 * @post $none
	 */
	protected void processOtherEvent(SimEvent ev) {
		if (ev == null) {
			Log.printLine(getName() + ".processOtherEvent(): " + "Error - an event is null.");
			return;
		}

		Log.printLine(getName() + ".processOtherEvent(): " + "Error - event unknown by Monitoring.");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.cloudbus.cloudsim.core.SimEntity#shutdownEntity()
	 */
	@Override
	public void shutdownEntity() {
		Log.printLine(getName() + " is shutting down...");
	}

	/**
	 * @param newEvent
	 *            the {@link SimEvent} to be added to queue
	 */
	@SuppressWarnings("unchecked")
	public void addMonitoringEvent(SimEvent newEvent) {
		if (newEvent.getSource() == getId() && getMonitoringEvent() != newEvent) {
			/*
			 * new Monitoring Event added to future-queue ... save event for
			 * modifications necessary when other monitoring-trigger events are
			 * added
			 */
			setMonitoringEvent(newEvent);
		}
		// Event-Tags triggering resource monitoring
		else if (newEvent.getTag() == CloudSimTags.CLOUDLET_SUBMIT
				|| newEvent.getTag() == CloudSimTags.CLOUDLET_RETURN) {
			if (getMonitoringEvent().eventTime() < FederatedCloudSim.clock()) {
				List<SimEvent> eventList = new LinkedList<SimEvent>();
				eventList.add(newEvent);
				sendNow(getId(), FederatedCloudSimTags.MONITORING, eventList);
			} else
				((LinkedList<SimEvent>) getMonitoringEvent().getData()).add(newEvent);
		}
		// SLA_CHECK Events triggering resource monitoring
		else if (newEvent.getTag() == FederatedCloudSimTags.SLA_CHECK) {
			List<SimEvent> eventList = new LinkedList<SimEvent>();
			eventList.add(newEvent);
			FederatedCloudSim.send(getId(), getId(), newEvent.eventTime() + 1, FederatedCloudSimTags.MONITORING,
					eventList);
		}
	}

	private SimEvent getMonitoringEvent() {
		return this.monitoringEvent;
	}

	private void setMonitoringEvent(SimEvent last) {
		this.monitoringEvent = last;
	}

	public long getTic() {
		return tic;
	}

	public boolean isLogInitialAllocation() {
		return logInitalAllocation;
	}

	public void setLogInitialAllocation(boolean initialOn) {
		logInitalAllocation = initialOn;
	}

	public void setTotalTics(long totalTics) {
		this.totalTics = totalTics;
	}

	public long getTotalTics() {
		return this.totalTics;
	}
}
