/**
 * 
 */
package com.materna.federatedcloudsim.monitoring;

import com.materna.federatedcloudsim.CloudServiceProvider;
import com.materna.federatedcloudsim.FederatedDatacenter;

/**
 * @author Marc Spohr
 *
 */
public class ResourcesBeanCsp extends ResourcesBean
{
	private final CloudServiceProvider csp;

	/**
	 * @param csp the {@link CloudServiceProvider} this resources Object belongs to 
	 */
	public ResourcesBeanCsp(CloudServiceProvider csp)
	{
		super();
		this.csp = csp; 
	}

	/* (non-Javadoc)
	 * @see com.materna.federatedcloudsim.monitoring.ResourcesBean#updatePe()
	 */
	@Override
	protected void updatePeCount()
	{
		int peCount = 0;
		for(FederatedDatacenter dc : getCsp().getDcList())
			peCount += dc.getResources().getPeCount().getTotal();
		getPeCount().setTotal(peCount);
	}

	/* (non-Javadoc)
	 * @see com.materna.federatedcloudsim.monitoring.ResourcesBean#updateMips()
	 */
	@Override
	protected void updateMips()
	{
		double mips = 0;
		for(FederatedDatacenter dc : getCsp().getDcList())
			mips += dc.getResources().getMips().getTotal();
		getMips().setTotal(mips);
	}

	/* (non-Javadoc)
	 * @see com.materna.federatedcloudsim.monitoring.ResourcesBean#updateMipsUsed()
	 */
	@Override
	protected void updateMipsUsed()
	{
		double mipsUsed = 0;
		for(FederatedDatacenter dc : getCsp().getDcList())
			mipsUsed += dc.getResources().getMips().getUsed();
		getMips().setUsed(mipsUsed);
	}

	/* (non-Javadoc)
	 * @see com.materna.federatedcloudsim.monitoring.ResourcesBean#updateRam()
	 */
	@Override
	protected void updateRam()
	{
		int ram = 0;
		for(FederatedDatacenter dc : getCsp().getDcList())
			ram += dc.getResources().getRam().getTotal();
		getRam().setTotal(ram);
	}

	/* (non-Javadoc)
	 * @see com.materna.federatedcloudsim.monitoring.ResourcesBean#updateRamUsed()
	 */
	@Override
	protected void updateRamUsed()
	{
		int ramUsed = 0;
		for(FederatedDatacenter dc : getCsp().getDcList())
			ramUsed += dc.getResources().getRam().getUsed();
		getRam().setUsed(ramUsed);
	}
	
	protected void updateConfiguredRam()
	{
		int configuredRam = 0;
		for(FederatedDatacenter dc : getCsp().getDcList())
			configuredRam += dc.getResources().getConfiguredRam().getTotal();
		getConfiguredRam().setTotal(configuredRam);
	}

	/* (non-Javadoc)
	 * @see com.materna.federatedcloudsim.monitoring.ResourcesBean#updateBw()
	 */
	@Override
	protected void updateBw()
	{
		long bw = Long.MAX_VALUE;
		for(FederatedDatacenter dc : getCsp().getDcList())
			if(bw > dc.getResources().getBw().getTotal())
				bw = dc.getResources().getBw().getTotal();
		getBw().setTotal(bw);
	}

	/* (non-Javadoc)
	 * @see com.materna.federatedcloudsim.monitoring.ResourcesBean#updateBwUsed()
	 */
	@Override
	protected void updateBwUsed()
	{
		long bwUsed = 0;
		for(FederatedDatacenter dc : getCsp().getDcList())
			if(bwUsed < dc.getResources().getBw().getUsed())
				bwUsed = dc.getResources().getBw().getUsed();
		getBw().setUsed(bwUsed);
	}

	/* (non-Javadoc)
	 * @see com.materna.federatedcloudsim.monitoring.ResourcesBean#updateHostStorage()
	 */
	@Override
	protected void updateHostStorage()
	{
		long hostStorage = 0;
		for(FederatedDatacenter dc : getCsp().getDcList())
			hostStorage += dc.getResources().getHostStorage().getTotal();
		getHostStorage().setTotal(hostStorage);
	}

	/* (non-Javadoc)
	 * @see com.materna.federatedcloudsim.monitoring.ResourcesBean#updateHostStorageUsed()
	 */
	@Override
	protected void updateHostStorageUsed()
	{
		long hostStorageUsed = 0;
		for(FederatedDatacenter dc : getCsp().getDcList())
			hostStorageUsed += dc.getResources().getHostStorage().getUsed();
		getHostStorage().setUsed(hostStorageUsed);
	}

	/* (non-Javadoc)
	 * @see com.materna.federatedcloudsim.monitoring.ResourcesBean#updateVmCount()
	 */
	@Override
	protected void updateVmCount()
	{
		int vmCount = 0;
		for(FederatedDatacenter dc : getCsp().getDcList())
			vmCount += dc.getResources().getVmCount().getTotal();
		getVmCount().setTotal(vmCount);
	}

	/* (non-Javadoc)
	 * @see com.materna.federatedcloudsim.monitoring.ResourcesBean#updateCloudletCount()
	 */
	@Override
	protected void updateCloudletCount()
	{
		int cloudletCount = 0;
		for(FederatedDatacenter dc : getCsp().getDcList())
			cloudletCount += dc.getResources().getCloudletCount().getTotal();
		getCloudletCount().setTotal(cloudletCount);
	}

	/* (non-Javadoc)
	 * @see com.materna.federatedcloudsim.monitoring.ResourcesBean#triggerParentUpdate()
	 */
	protected void triggerParentUpdate()
	{
		//no parent entity
	}
	
	/**
	 * @return the csp
	 */
	public CloudServiceProvider getCsp()
	{
		return csp;
	}

}
