/**
 * 
 */
package com.materna.federatedcloudsim.scheduling.csp;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.cloudbus.cloudsim.Host;

import com.materna.federatedcloudsim.FederatedDatacenter;
import com.materna.federatedcloudsim.FederatedHost;
import com.materna.federatedcloudsim.FederatedVm;
import com.materna.federatedcloudsim.federation.sla.Sla;
import com.materna.federatedcloudsim.lists.FederatedVmList;
import com.materna.federatedcloudsim.scheduling.datacenter.FederatedVmAllocationPolicy;
//import com.materna.federatedcloudsim.scheduling.datacenter.FederatedVmAllocationPolicyNF;

/**
 * @author Marc Spohr
 *
 */
public abstract class FederatedDatacenterBrokerStrategy {
	private FederatedDatacenterBroker dcBroker;

	/**
	 * @param sla
	 *            the {@link Sla} of the Service to be scheduled
	 * @param dcIdsList
	 *            the list of Datacenter ids
	 * @return the {@link FederatedDatacenter} this service should be scheduled to
	 */
	public abstract FederatedDatacenter getDc(Sla sla, List<Integer> dcIdsList);

	public abstract FederatedDatacenter getDc(FederatedVm vm);

	public abstract Map<FederatedVm, FederatedHost> findPossibleHost(FederatedVm fedVm);

	public void configure(Map<String, String> params) {
	}

	/**
	 * @return the {@link FederatedDatacenterBroker}
	 */
	public FederatedDatacenterBroker getDcBroker() {
		return dcBroker;
	}

	/**
	 * @param dcBroker
	 *            the {@link FederatedDatacenterBroker} to set
	 */
	public void setDcBroker(FederatedDatacenterBroker dcBroker) {
		this.dcBroker = dcBroker;
	}

	public ArrayList<FederatedVm> findVmsForMigration(ArrayList<FederatedVm> vmList, FederatedDatacenter dc) {

		ArrayList<FederatedVm> retList = new ArrayList<FederatedVm>();
		retList.addAll(vmList);
		// No strategy on broker level: Return the inputlist.
		return retList;
	}

	public Map<FederatedVm, FederatedHost> findVmsForMigration(FederatedVm fedVms)
			throws UnsupportedOperationException {
		throw new UnsupportedOperationException();
	}

	/**
	 * Checks wether a host should be turned on or not.
	 * 
	 * @param vmList
	 * @return
	 */
	public Map<FederatedVm, FederatedHost> checkTurnedOffHosts(List<FederatedVm> vmList) {
		List<FederatedVm> vmsToAllocate = new ArrayList<FederatedVm>();
		vmsToAllocate.addAll(vmList);
		Map<FederatedVm, FederatedHost> ret = new TreeMap<FederatedVm, FederatedHost>();

		List<FederatedVm> vms2delete = new ArrayList<FederatedVm>();

		for (FederatedDatacenter fedDc : getDcBroker().getCsp().getDcList()) {
			List<FederatedHost> offList = fedDc.getHostOffList();

			if (offList != null)
				for (FederatedHost fedHost : offList) {

					double mips = 0.0;
					int ram = 0;
					int allocRam = 0;
					long bw = 0;

					for (FederatedVm fedVm : vmsToAllocate) {
						mips += fedVm.getResources().getMips().getUsed();
						ram += fedVm.getResources().getRam().getUsed();
						allocRam += fedVm.getRam();
						bw += fedVm.getCurrentRequestedBw();

						if (fedHost.isSuitableForVmOFF(FederatedVmAllocationPolicy.PERC_TH, mips, ram, allocRam,
								fedVm.getResources().getPeCount().getTotal(), bw)) {
							vms2delete.add(fedVm);
							ret.put(fedVm, fedHost);
						} else
							break;
					}

					vmsToAllocate.removeAll(vms2delete);
				}
		}

		return ret;
	}
}
