package com.materna.federatedcloudsim.scheduling.csp;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import org.cloudbus.cloudsim.Host;
import org.cloudbus.cloudsim.Pe;
import org.cloudbus.cloudsim.Vm;

import com.materna.federatedcloudsim.FederatedDatacenter;
import com.materna.federatedcloudsim.FederatedHost;
import com.materna.federatedcloudsim.FederatedVm;
import com.materna.federatedcloudsim.federation.sla.Sla;
import com.materna.federatedcloudsim.monitoring.Monitoring;
import com.materna.federatedcloudsim.scheduling.datacenter.FederatedVmAllocationPolicy;
//import com.sun.corba.se.impl.ior.GenericTaggedComponent;

/**
 * First Fit implementation of {@link FederatedDatacenterBrokerStrategy}
 * 
 * @author Marcel Krueger
 *
 */
public class FederatedDatacenterBrokerStrategyFirstFit extends FederatedDatacenterBrokerStrategy {

	public FederatedDatacenterBrokerStrategyFirstFit() {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.materna.federatedcloudsim.scheduling.datacenter.FederatedDatacenterBrokerStrategy#getDcId(com.materna.federatedcloudsim.federation.Sla)
	 */
	@Override
	public FederatedDatacenter getDc(Sla sla, List<Integer> dcIdsList) {
		FederatedDatacenter ret = null;

		// TODO Maybe without if...check should be done at CloudBroker-Level!?!
		if (getDcBroker().getCsp().provides(sla.getService()) && dcIdsList.size() > 0) {

			boolean suitable = false;

			for (FederatedDatacenter fedDc : getDcBroker().getCsp().getDcList()) {

				for (Host host : fedDc.getHostOnList()) {
					FederatedHost fedHost = (FederatedHost) host;
					int pesNumber = Integer.parseInt(sla.getSlos().get("pesNumber"));
					int ram = Integer.parseInt(sla.getSlos().get("memory")) / pesNumber;
					// TODO changed
					Vm cacheVm = new Vm(-1, -1, 0, pesNumber, ram, 0, 0, "", null);
					if (suitable = fedHost.isSuitableForVm(cacheVm))
						break;
				}

				if (suitable)
					ret = fedDc;
			}
		}

		return ret;
	}

	@Override
	public FederatedDatacenter getDc(FederatedVm vm) {
		Set<FederatedDatacenter> dcList = new TreeSet<FederatedDatacenter>();
		dcList.addAll(getDcBroker().getCsp().getDcList());

		// TODO ungleich dem alten DC? --> darf nicht geprft werden, da ja zeit vergangen ist und die vm wieder drauf passt...
//		if (!(vm.getDeAllocTic() < Monitoring.getInstance().getTic()) &&  vm.getFedHostMigHistory().size() > 0)
//			dcList.remove(vm.getFedHostMigHistory().get(vm.getFedHostMigHistory().size() - 1).getDatacenter());

		for (FederatedDatacenter dc : dcList) {
			FederatedHost host = ((FederatedVmAllocationPolicy) dc.getVmAllocationPolicy()).findDestinationHost(vm);
			if (host != null) {
				return dc;
			}
		}

		return null;
	}

	public ArrayList<FederatedVm> findVmsForMigration(ArrayList<FederatedVm> vmList, FederatedDatacenter dc) {

		ArrayList<FederatedVm> retList = new ArrayList<FederatedVm>();
		retList.addAll(vmList);
		// No strategy on broker level: Return the inputlist.
		return retList;
	}

	/**
	 * Finds a possible destination host for fedVm. Returns a Map: Key - FederatedVm to Move (also VMs which have to be migrated too); Value -
	 * destination host
	 * 
	 * Here: Uses getDC(FederatedVm) and VmAllovationPolicy to find a destination host. No other VMs will be returned.
	 * 
	 * @return Returns a Map (non null); one key is the vm which has been used as method parameter
	 */
	@Override
	public Map<FederatedVm, FederatedHost> findPossibleHost(FederatedVm fedVm) {
		FederatedHost possibleHost = null;
		TreeMap<FederatedVm, FederatedHost> result = new TreeMap<FederatedVm, FederatedHost>();
		FederatedDatacenter fedDc = getDc(fedVm);

		if (fedDc != null) {
			possibleHost = ((FederatedVmAllocationPolicy) fedDc.getVmAllocationPolicy()).findDestinationHost(fedVm);
			if (possibleHost != null)
				result.put(fedVm, possibleHost);
		}

		return result;
	}

	@Override
	public Map<FederatedVm, FederatedHost> checkTurnedOffHosts(List<FederatedVm> vmList) {
		List<FederatedVm> vmsToAllocate = new ArrayList<FederatedVm>();
		vmsToAllocate.addAll(vmList);
		Map<FederatedVm, FederatedHost> ret = new TreeMap<FederatedVm, FederatedHost>();

		List<FederatedVm> vms2delete = new ArrayList<FederatedVm>();

		for (FederatedDatacenter fedDc : getDcBroker().getCsp().getDcList()) {
			List<FederatedHost> offList = fedDc.getHostOffList();

			if (offList != null)
				for (FederatedHost fedHost : offList) {

					double mips = 0.0;
					int ram = 0;
					int allocRam = 0;
					long bw = 0;

					for (FederatedVm fedVm : vmsToAllocate) {
						mips += fedVm.getResources().getMips().getUsed();
						ram += fedVm.getResources().getRam().getUsed();
						allocRam += fedVm.getRam();
						bw += fedVm.getCurrentRequestedBw();

						if (fedHost.isSuitableForVmOFF(FederatedVmAllocationPolicy.PERC_TH, mips, ram, allocRam,
								fedVm.getResources().getPeCount().getTotal(), bw)) {
							vms2delete.add(fedVm);
							ret.put(fedVm, fedHost);
						} else
							break;
					}

					vmsToAllocate.removeAll(vms2delete);
				}
		}

		return ret;
	}

}