package com.materna.federatedcloudsim.scheduling.csp;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import org.cloudbus.cloudsim.Host;
import org.cloudbus.cloudsim.Vm;

import com.materna.federatedcloudsim.FederatedDatacenter;
import com.materna.federatedcloudsim.FederatedHost;
import com.materna.federatedcloudsim.FederatedVm;
import com.materna.federatedcloudsim.federation.sla.Sla;
import com.materna.federatedcloudsim.monitoring.Monitoring;
import com.materna.federatedcloudsim.scheduling.datacenter.FederatedVmAllocationPolicy;

public class FederatedDatacenterBrokerStrategyMinimizeMigrationLocalBestDangerAverage
		extends FederatedDatacenterBrokerStrategy {

	public FederatedDatacenterBrokerStrategyMinimizeMigrationLocalBestDangerAverage() {
		// TODO Auto-generated constructor stub
	}

	@Override
	public FederatedDatacenter getDc(Sla sla, List<Integer> dcIdsList) {
		FederatedDatacenter ret = null;

		// TODO Maybe without if...check should be done at CloudBroker-Level!?!
		if (getDcBroker().getCsp().provides(sla.getService()) && dcIdsList.size() > 0) {

			boolean suitable = false;

			for (FederatedDatacenter fedDc : getDcBroker().getCsp().getDcList()) {

				for (Host host : fedDc.getHostList()) {
					FederatedHost fedHost = (FederatedHost) host;
					int pesNumber = Integer.parseInt(sla.getSlos().get("pesNumber"));
					int ram = Integer.parseInt(sla.getSlos().get("memory")) / pesNumber;
					// TODO changed
					if (suitable = fedHost.isSuitableForVm(new Vm(-1, -1, 0, pesNumber, ram, 0, 0, "", null)))
						break;
				}

				if (suitable)
					ret = fedDc;
			}
		}

		return ret;
	}

	public FederatedDatacenter getDc(FederatedVm fedVm) {
		Set<FederatedDatacenter> dcList = new TreeSet<FederatedDatacenter>();
		dcList.addAll(getDcBroker().getCsp().getDcList());

		// TODO ungleich dem alten DC?
//		if (fedVm.getDeAllocTic() < Monitoring.getInstance().getTic() && fedVm.getFedHostMigHistory().size() > 0)
//			dcList.remove(fedVm.getFedHostMigHistory().get(fedVm.getFedHostMigHistory().size() - 1).getDatacenter());

		ArrayList<FederatedHost> allHostList = new ArrayList<FederatedHost>();
		for (FederatedDatacenter dc : dcList) {
			List<FederatedHost> fedHostList = dc.getHostOnList();
			allHostList.addAll(fedHostList);
		}
		List<FederatedHost> sortedHostList = sortHostsAscendingByPercentalSlaDangerAverage(allHostList);

		for (FederatedHost fedHost : sortedHostList) {
//			if (fedHost.isSuitableForVm(fedVm)) {
			if (fedHost.isSuitableForVm(fedVm, 0.8f)) {
				return fedHost.getDatacenter();
			}
		}
		return null;
	}

	private ArrayList<FederatedHost> sortHostsAscendingByPercentalSlaDangerAverage(ArrayList<FederatedHost> list2Sort) {

		Float meanPercentalDanger = 0f;
		ArrayList<FederatedHost> sortedHostList = new ArrayList<>();

		while (!list2Sort.isEmpty()) {
			FederatedHost fewestDangeredHost = null;
			Float fewestDanger = Float.MAX_VALUE;
			for (FederatedHost sourceHost : list2Sort) {
				for (int i = 0; i < sourceHost.getVmList().size(); i++) {
					FederatedVm fedVm = (FederatedVm) sourceHost.getVmList().get(i);
					Float percentalDanger = fedVm.getActualDownTime() / fedVm.getMaxDowntime();
					meanPercentalDanger = (meanPercentalDanger * i + percentalDanger) / (i + 1);
				}
				if (meanPercentalDanger < fewestDanger) {
					fewestDanger = meanPercentalDanger;
					fewestDangeredHost = sourceHost;
				}
			}
			sortedHostList.add(fewestDangeredHost);
			list2Sort.remove(fewestDangeredHost);
		}

		return sortedHostList;
	}

	public ArrayList<FederatedVm> findVmsForMigration(ArrayList<FederatedVm> vmList, FederatedDatacenter dc) {

		// calculate the resources that needs to be freed
		int ramToFree = 0;
		double mipsToFree = 0;

		for (FederatedVm vm : vmList) {
			ramToFree += vm.getResources().getRam().getUsed();
			mipsToFree += vm.getResources().getMips().getUsed();
		}

		// get all vms from the endangerd host
		ArrayList<FederatedVm> fedVmList = new ArrayList<>();
		FederatedHost fedHost = (FederatedHost) vmList.get(0).getHostHistory()
				.get(vmList.get(0).getHostHistory().size() - 1);
		FederatedVmAllocationPolicy policy = (FederatedVmAllocationPolicy) fedHost.getDatacenter()
				.getVmAllocationPolicy();
		fedVmList.addAll(fedHost.getVmList());

		// sort all vms according to their ressource usage, high ressource usage
		// first
		ArrayList<FederatedVm> ressourceUsageSortedVmList = sortVmsDescendingByResourceUsage(fedVmList);

		ArrayList<FederatedVm> retCacheList = new ArrayList<FederatedVm>();
		ArrayList<FederatedVm> retList = new ArrayList<FederatedVm>();

		// choose the vms from the slaDangerSortedVmList which Resources sum up
		// to
		// the needed Resources
		chooseVms(ressourceUsageSortedVmList, retCacheList, ramToFree, mipsToFree);

		// separate the vms who remain unchanged for migration from those which
		// are chosen to replace formerly picked vms and deallocate the
		// replacing
		List<FederatedVm> unchangedVms = new ArrayList<>();
		List<FederatedVm> changedVms = new ArrayList<>();
		for (FederatedVm chosenVm : retCacheList) {
			if (chosenVm.getHost() != null) {
				changedVms.add(chosenVm);
				policy.deallocateHostForVm(chosenVm);
				fedHost.getResources().update();
			} else
				unchangedVms.add(chosenVm);
		}

		// get the vms that need to be allocated, which are the replaced,
		// formerly picked vms
		List<FederatedVm> toBeAllocatedVms = new ArrayList<>(vmList);
		toBeAllocatedVms.removeAll(unchangedVms);

		// reallocate formerly picked vms back on their original host
		for (FederatedVm vm : toBeAllocatedVms) {
			if (fedHost.vmCreate(vm)) {
				policy.getVmTable().put(vm.getUid(), fedHost);
			} else {
				retList.add(vm);
			}
			fedHost.getResources().update();
		}
		retList.addAll(retCacheList);
		return retList;
	}

	private void chooseVms(ArrayList<FederatedVm> inputList, ArrayList<FederatedVm> outputList, int ram2Free,
			double mips2Free) {
		// choose the vms from the input List which Resources sum up to
		// the parameter resources, delete them from the inputlist and put
		// them in the outputlist
		int ramUsed = 0;
		int mipsUsed = 0;
		ArrayList<FederatedVm> vms2Remove = new ArrayList<>();

		for (int i = 0; i < inputList.size(); i++) {
			if (ram2Free > ramUsed && mips2Free > mipsUsed) {
				if (FederatedVmAllocationPolicy.checkMigratable(inputList.get(i))) {
					outputList.add(inputList.get(i));
					ramUsed += inputList.get(i).getResources().getRam().getUsed();
					mipsUsed += inputList.get(i).getResources().getMips().getUsed();
					vms2Remove.add(inputList.get(i));
				}
			} else {
				break;
			}
		}
		inputList.removeAll(vms2Remove);
	}

	private ArrayList<FederatedVm> sortVmsDescendingByResourceUsage(ArrayList<FederatedVm> list2Sort) {

		Double ressourceUse = 0d;
		ArrayList<FederatedVm> sortedVmList = new ArrayList<>();

		while (!list2Sort.isEmpty()) {
			FederatedVm highestResourceUsageVm = null;
			Double highestUsage = Double.MIN_VALUE;
			for (FederatedVm vm : list2Sort) {
				ressourceUse = vm.getResources().getRam().getUsed() + vm.getResources().getMips().getUsed();
				if (ressourceUse == 0.0) {
					ressourceUse = Double.MIN_VALUE;
				}
				if (ressourceUse >= highestUsage) {
					highestUsage = ressourceUse;
					highestResourceUsageVm = vm;
				}
			}
			// remove all Gold-SLAs
			if (highestResourceUsageVm.getFirstCloudlet().getSr().getSla().getLevel() != Sla.LEVEL_GOLD) {
				sortedVmList.add(highestResourceUsageVm);
			}
			list2Sort.remove(highestResourceUsageVm);
		}
		return sortedVmList;
	}

	/**
	 * Finds a possible destination host for fedVm. Returns a Map: Key -
	 * FederatedVm to Move (also VMs which have to be migrated too); Value -
	 * destination host
	 * 
	 * Here: Uses getDC(FederatedVm) and VmAllovationPolicy to find a
	 * destination host. No other VMs will be returned.
	 * 
	 * @return Returns a Map (non null); one key is the vm which has been used
	 *         as method parameter
	 */
	@Override
	public Map<FederatedVm, FederatedHost> findPossibleHost(FederatedVm fedVm) {
		FederatedHost possibleHost = null;
		TreeMap<FederatedVm, FederatedHost> result = new TreeMap<FederatedVm, FederatedHost>();
		FederatedDatacenter fedDc = getDc(fedVm);

		if (fedDc != null) {
			possibleHost = ((FederatedVmAllocationPolicy) fedDc.getVmAllocationPolicy()).findDestinationHost(fedVm);
			if (possibleHost != null)
				result.put(fedVm, possibleHost);
		}
		return result;
	}

}
