package com.materna.federatedcloudsim.scheduling.datacenter;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.cloudbus.cloudsim.Host;

import com.materna.federatedcloudsim.FederatedHost;
import com.materna.federatedcloudsim.FederatedVm;
import com.materna.federatedcloudsim.core.FederatedCloudSim;
import com.materna.federatedcloudsim.lists.FederatedHostList;
import com.materna.federatedcloudsim.lists.FederatedVmList;

public class FederatedVmAllocationPolicyHPGOP extends FederatedVmAllocationPolicy{

	public FederatedVmAllocationPolicyHPGOP(List<? extends Host> list) {
		super(list);
		// TODO Auto-generated constructor stub
	}
	
	public FederatedVmAllocationPolicyHPGOP() {
		super();
		// TODO Auto-generated constructor stub
	}

	@Override
	public ArrayList<FederatedVm> findVmsForMigration(FederatedHost endangeredHost) {

		double totalRam = endangeredHost.getResources().getRam().getTotal();
		double ramTH = totalRam * PERC_TH;
		double usedRam = endangeredHost.getResources().getRam().getUsed();
		double ramToFree = usedRam - ramTH;

		double totalMips = endangeredHost.getTotalMips();
		double mipsTH = totalMips * PERC_TH;
		double usedMips = (endangeredHost.getResources().getMips().getUsed());
		double mipsToFree = usedMips - mipsTH;

		boolean lessCPU = (mipsToFree > 0);
		boolean lessRam = (ramToFree > 0);

		// Sort the VMList

		Collection<? extends FederatedVm> hostVmList = endangeredHost.getVmList();
		List<FederatedVm> tempList = new ArrayList<FederatedVm>();
		tempList.addAll(hostVmList);

		if (lessCPU) {
			FederatedVmList.sortDescendingByPotentialGrowthCpu(tempList);
		} else {
			FederatedVmList.sortDescendingByPotentialGrowthRam(tempList);
		}

		// ArrayList<FederatedVm> sortedVmList = new ArrayList<FederatedVm>();
		// sortedVmList.addAll(tempList);
		ArrayList<FederatedVm> vmsToMove = new ArrayList<FederatedVm>();

		// Find minimum amount of VMs for migration
		double ramFreeValue = 0;
		double mipsFreeValue = 0;

		if (lessRam) {

			for (int i = 0; i < tempList.size() && ramFreeValue < ramToFree; i++) {

				// If the VM used resources are higher than 0
				// and the vm is movable
				if (tempList.get(i).getResources().getRam().getUsed() > 0
						&& !getEpc().getNonMovableVms().contains(tempList.get(i))) {
					ramFreeValue = ramFreeValue + tempList.get(i).getResources().getRam().getUsed();
					mipsFreeValue = mipsFreeValue + tempList.get(i).getResources().getMips().getUsed();
					vmsToMove.add(tempList.get(i));
				}
			}
		}

		if (lessCPU) {

			for (int i = 0; i < tempList.size() && mipsFreeValue < mipsToFree; i++) {

				// If the VM used resources are higher than resources to
				// free and the vm is movable
				if (tempList.get(i).getResources().getMips().getUsed() > 0
						&& !getEpc().getNonMovableVms().contains(tempList.get(i))
						&& !tempList.contains(tempList.get(i))) {
					ramFreeValue = ramFreeValue + tempList.get(i).getResources().getRam().getUsed();
					mipsFreeValue = mipsFreeValue + tempList.get(i).getResources().getMips().getUsed();
					vmsToMove.add(tempList.get(i));
				}
			}
		}
		return vmsToMove;
	}

	@Override
	public FederatedHost findDestinationHost(FederatedVm vm) {

		// BestFit by most occupied resource

		
		Collection<? extends FederatedHost> hostList = getHostList();
		ArrayList<FederatedHost> tempHosts = new ArrayList<FederatedHost>();
		tempHosts.addAll(hostList);

		if(vm.getTotalUtilizationOfCpu(FederatedCloudSim.clock()) > (double) vm.getResources().getRam().getUsed() / (double) vm.getResources().getRam().getTotal()) {
			FederatedHostList.sortAscendingByOcValue(tempHosts);
		}	else {
			FederatedHostList.sortAscendingByOcValue(tempHosts);
		}
		for (int i = 0; i < tempHosts.size() ; i++) {
			if (tempHosts.get(i).isSuitableForVm(vm, PERC_TH) && tempHosts.get(i) != vm.getHost()) {
				return tempHosts.get(i);
			}
		}

		return null;
	}

}
