package com.materna.federatedcloudsim.scheduling.datacenter;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.cloudbus.cloudsim.Host;
import org.cloudbus.cloudsim.Vm;

import com.materna.federatedcloudsim.FederatedDatacenter;
import com.materna.federatedcloudsim.FederatedHost;
import com.materna.federatedcloudsim.FederatedVm;
import com.materna.federatedcloudsim.core.FederatedCloudSim;
import com.materna.federatedcloudsim.federation.sla.Sla;
import com.materna.federatedcloudsim.lists.FederatedHostList;
import com.materna.federatedcloudsim.lists.FederatedVmList;

public class FederatedVmAllocationPolicyMMWF extends FederatedVmAllocationPolicy {

	public FederatedVmAllocationPolicyMMWF(List<? extends Host> list) {
		super(list);
	}

	public FederatedVmAllocationPolicyMMWF() {
		super();
	}

	@Override
	public ArrayList<FederatedVm> findVmsForMigration(FederatedHost endangeredHost) {

		double totalRam = endangeredHost.getResources().getRam().getTotal();
		double ramTH = totalRam * PERC_TH;
		double usedRam = endangeredHost.getResources().getRam().getUsed();
		double ramToFree = usedRam - ramTH;

		double totalMips = endangeredHost.getTotalMips();
		double mipsTH = totalMips * PERC_TH;
		double usedMips = (endangeredHost.getResources().getMips().getUsed());
		double mipsToFree = usedMips - mipsTH;

		boolean lessCPU = (mipsToFree > 0);
		boolean lessRam = (ramToFree > 0);

		// Sort the VMList

		Collection<? extends FederatedVm> hostVmList = endangeredHost.getVmList();
		List<FederatedVm> tempList = new ArrayList<FederatedVm>();
		tempList.addAll(hostVmList);

		List<FederatedVm> goldList = new ArrayList<FederatedVm>();
		// remove all Gold-SLAs
		for (int i = 0; i < tempList.size(); i++) {
			if (tempList.get(i).getFirstCloudlet().getSr().getSla().getLevel() == Sla.LEVEL_GOLD)
				goldList.add(tempList.get(i));
		}
		tempList.removeAll(goldList);

		if (lessCPU) {
			FederatedVmList.sortAscendingByUsedMips(tempList);
		} else {
			FederatedVmList.sortAscendingByUsedRam(tempList);
		}

		ArrayList<FederatedVm> vmsToMove = new ArrayList<FederatedVm>();

		// Find minimum amount of VMs for migration
		double ramFreeValue = 0;
		double mipsFreeValue = 0;

		if (lessRam) {

			while (ramFreeValue < ramToFree && !tempList.isEmpty()) {
				boolean notEnoughVms = false;

				for (int i = 0; i < tempList.size(); i++) {

					// If the VM used resources are higher than resources to
					// free and the vm is movable
					if ((tempList.get(i).getResources().getRam().getUsed() > (ramToFree - ramFreeValue))
							&& checkMigratable(tempList.get(i))) {
						ramFreeValue = ramFreeValue + tempList.get(i).getResources().getRam().getUsed();
						mipsFreeValue = mipsFreeValue + tempList.get(i).getResources().getMips().getUsed();
						vmsToMove.add(tempList.get(i));
						break;
					} else { // VM used resources are lower than resources to
								// free
						boolean iterateFinished = false;

						if (i == tempList.size() - 1) { // If last VM reached in
														// list -> take last vm
														// and search for
														// further vms
							if (checkMigratable(tempList.get(i))) {
								ramFreeValue = ramFreeValue + tempList.get(i).getResources().getRam().getUsed();
								mipsFreeValue = mipsFreeValue + tempList.get(i).getResources().getMips().getUsed();
								vmsToMove.add(tempList.get(i));
								tempList.remove(i);
							} else {
								for (int j = 1; i - j >= 0 && !iterateFinished; j++) {
									if (checkMigratable(tempList.get(i - j))) {
										iterateFinished = true;
										ramFreeValue = ramFreeValue
												+ tempList.get(i - j).getResources().getRam().getUsed();
										mipsFreeValue = mipsFreeValue
												+ tempList.get(i - j).getResources().getMips().getUsed();
										vmsToMove.add(tempList.get(i - j));
										tempList.remove(i - j);
									}
								}

								if (!iterateFinished) {
									notEnoughVms = true;
									break;
								}

							}
						}
					}

				}

				if (notEnoughVms)
					break;

			}
		}

		if (lessCPU) {
			int j = tempList.size();
			while (mipsFreeValue < mipsToFree && j != 0) {
				boolean notEnoughVms = false;

				for (int i = 0; i < tempList.size(); i++) {

					// If the VM used resources are higher than resources to
					// free and the vm is movable
					if (tempList.get(i).getResources().getMips().getUsed() > (mipsToFree - mipsFreeValue)
							&& checkMigratable(tempList.get(i))) {
						ramFreeValue = ramFreeValue + tempList.get(i).getResources().getRam().getUsed();
						mipsFreeValue = mipsFreeValue + tempList.get(i).getResources().getMips().getUsed();
						vmsToMove.add(tempList.get(i));
						break;
					} else { // VM used resources are lower than resources to
								// free

						boolean iterateFinished = false;

						if ((i == tempList.size() - 1)) { // If
							// last
							// VM
							// reached
							// in
							// list -> take last vm
							// and search for
							// further vms
							if (checkMigratable(tempList.get(i))) {
								ramFreeValue = ramFreeValue + tempList.get(i).getResources().getRam().getUsed();
								mipsFreeValue = mipsFreeValue + tempList.get(i).getResources().getMips().getUsed();
								vmsToMove.add(tempList.get(i));
								tempList.remove(i);
							} else {
								for (int f = 1; i - f >= 0 && !iterateFinished; f++) {
									if (checkMigratable(tempList.get(i - f))) {
										iterateFinished = true;
										ramFreeValue = ramFreeValue
												+ tempList.get(i - f).getResources().getRam().getUsed();
										mipsFreeValue = mipsFreeValue
												+ tempList.get(i - f).getResources().getMips().getUsed();
										vmsToMove.add(tempList.get(i - f));
										tempList.remove(i - f);
									}

								}

								if (!iterateFinished) {
									notEnoughVms = true;
									break;
								}

							}
						}
					}

				}
				j--;

				if (notEnoughVms)
					break;
			}

		}
		return vmsToMove;
	}

	@SuppressWarnings("static-access")
	@Override
	public FederatedHost findDestinationHost(FederatedVm vm) {

		// BestFit by most occupied resource

		Collection<? extends FederatedHost> hostList = getHostList();
		ArrayList<FederatedHost> tempHosts = new ArrayList<FederatedHost>();
		tempHosts.addAll(hostList);

		// TODO mir erklren was das macht bitte! cpu nutzung > prozentuale ram auslastung???
		if (vm.getTotalUtilizationOfCpu(FederatedCloudSim.clock()) > (double) vm.getResources().getRam().getUsed()
				/ (double) vm.getResources().getRam().getTotal()) {
			FederatedHostList.sortAscendingByUsedMips(tempHosts);
		} else {
			FederatedHostList.sortDescendingByFreeRam(tempHosts);
		}
		for (int i = 0; i < tempHosts.size(); i++) {
			if (tempHosts.get(i).isSuitableForVm(vm, PERC_TH) && tempHosts.get(i) != vm.getHost()) {
				return tempHosts.get(i);
			}
		}

		return null;
	}

}
