package com.materna.federatedcloudsim.scheduling.federation;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import com.materna.federatedcloudsim.CloudServiceProvider;

public class AuctionManager {

	private final CloudServiceProvider provider;
	private Set<Auction> startedAuctionsList;
	private Set<Auction> completedAuctionsList;
	private Set<Auction> rejectedAuctionsList;
	private Set<Auction> attendedAuctionsList;
	private Set<Auction> participatingAuctionsList;
	private Map<Auction, List<CloudServiceProvider>> auctionInvitationMap;
	private Map<Auction, Map<String, String>> auctionBidMap;
	private AuctionStrategy strategy;
	private int gainedVms;
	private int rejectedInvitations;
	private int rejectedLevelOne;
	private int rejectedLevelTwo;
	private int rejectedLevelThree;
	

	public AuctionManager(CloudServiceProvider provider, AuctionStrategy strategy) {
		this.provider = provider;
		this.strategy = strategy;
		startedAuctionsList = new TreeSet<>();
		completedAuctionsList = new TreeSet<>();
		attendedAuctionsList = new TreeSet<Auction>();
		participatingAuctionsList = new TreeSet<Auction>();
		rejectedAuctionsList = new TreeSet<Auction>();
		auctionInvitationMap = new TreeMap<Auction, List<CloudServiceProvider>>();
		auctionBidMap = new TreeMap<>();
		gainedVms = 0;
		rejectedInvitations = 0;
		rejectedLevelOne = 0;
		rejectedLevelTwo = 0;
		rejectedLevelThree = 0;
		getStrategy().setCb(provider.getCb());
		getStrategy().setAuctionManager(this);
	}

	public void startNewAuction(Object auctionObject) {
		Auction newAuction = new Auction(auctionObject, provider);
		startedAuctionsList.add(newAuction);
		
		Map<String, String> newBidMapEntry = new TreeMap<>();
		newBidMapEntry.put("Auction", String.valueOf(newAuction.getAuctionId()));
		newBidMapEntry.put("Load", String.valueOf(0d));
		newBidMapEntry.put("First", String.valueOf(0d));
		newBidMapEntry.put("Second", String.valueOf(0d));
		getAuctionBidMap().put(newAuction, newBidMapEntry);
		
		getStrategy().startInvitation(newAuction);
	}

	/**
	 * @return the provider
	 */
	public CloudServiceProvider getProvider() {
		return provider;
	}

	/**
	 * @return the startedAuctionsList
	 */
	public Set<Auction> getStartedAuctionList() {
		return startedAuctionsList;
	}

	/**
	 * @return the completedAuctionsList
	 */
	public Set<Auction> getCompletedAuctionsList() {
		return completedAuctionsList;
	}

	/**
	 * @return the strategy
	 */
	public AuctionStrategy getStrategy() {
		return strategy;
	}

	/**
	 * @param strategy
	 *            the strategy to set
	 */
	public void setStrategy(AuctionStrategy strategy) {
		this.strategy = strategy;
	}

	public Map<Auction, List<CloudServiceProvider>> getAuctionInvitationMap() {
		return auctionInvitationMap;
	}

	/**
	 * @return the attendedAuctionsList
	 */
	public Set<Auction> getAttendedAuctionsList() {
		return attendedAuctionsList;
	}

	/**
	 * @return the gainedVms
	 */
	public int getGainedVms() {
		return gainedVms;
	}

	public void increaseGainedVms() {
		gainedVms++;
	}

	/**
	 * @return the rejectedAuctionsList
	 */
	public Set<Auction> getRejectedAuctionsList() {
		return rejectedAuctionsList;
	}

	/**
	 * @return the participatingAuctionsList
	 */
	public Set<Auction> getParticipatingAuctionsList() {
		return participatingAuctionsList;
	}

	/**
	 * @return the rejectedInvitations
	 */
	public int getRejectedInvitations() {
		return rejectedInvitations;
	}
	
	public void increaseRejectedInvitations(){
		rejectedInvitations++;
	}
	
	/**
	 * @return the rejectedLevelOne
	 */
	public int getRejectedLevelOne() {
		return rejectedLevelOne;
	}
	
	public void increaseRejectedLevelOne(){
		rejectedLevelOne++;
	}
	
	/**
	 * @return the rejectedLevelTwo
	 */
	public int getRejectedLevelTwo() {
		return rejectedLevelTwo;
	}
	
	public void increaseRejectedLevelTwo(){
		rejectedLevelTwo++;
	}
	
	/**
	 * @return the rejectedLevelThree
	 */
	public int getRejectedLevelThree() {
		return rejectedLevelThree;
	}
	
	public void increaseRejectedLevelThree(){
		rejectedLevelThree++;
	}

	/**
	 * @return the auctionBidMap
	 */
	public Map<Auction, Map<String, String>> getAuctionBidMap() {
		return auctionBidMap;
	}
}
