package com.materna.federatedcloudsim.scheduling.federation;

import java.util.Map;
import java.util.TreeMap;

public abstract class AuctionStrategy extends CloudBrokerStrategyRelianceScope {

	private String auctionForm;
	private AuctionManager auctionManager;
	private BiddingStrategy biddingStrategy;
	private Map<String, String> biddingParameters;

	public AuctionStrategy(String auctionForm) {
		super(true);
		this.auctionForm = auctionForm;
		biddingParameters = new TreeMap<String, String>();
	}

	public abstract void startInvitation(Auction auction);

	public abstract boolean handleInvitation(Auction auction);

	public abstract void startAuction(Auction auction);

	public abstract void handleAuctionMessage(AuctionMessage message);

	public abstract void finishAuctionRound(Auction auction);

	public String getAuctionForm() {
		return auctionForm;
	}

	public AuctionManager getAuctionManager() {
		return auctionManager;
	}

	public void setAuctionManager(AuctionManager auctionManager) {
		this.auctionManager = auctionManager;
	}

	/**
	 * @return the biddingStrategy
	 */
	public BiddingStrategy getBiddingStrategy() {
		return biddingStrategy;
	}

	/**
	 * @param biddingStrategy
	 *            the biddingStrategy to set
	 */
	public void setBiddingStrategy(BiddingStrategy biddingStrategy) {
		this.biddingStrategy = biddingStrategy;
	}

	/**
	 * @return the biddingParameters
	 */
	public Map<String, String> getBiddingParameters() {
		return biddingParameters;
	}

	/**
	 * @param biddingParameters
	 *            the biddingParameters to set
	 */
	public void setBiddingParameters(Map<String, String> biddingParameters) {
		this.biddingParameters = biddingParameters;
	}
}
