package com.materna.federatedcloudsim.scheduling.federation;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import com.materna.federatedcloudsim.FederatedDatacenter;
import com.materna.federatedcloudsim.FederatedHost;

public class BiddingStrategy2RoundVmVariable extends BiddingStrategy {

	public BiddingStrategy2RoundVmVariable() {
		super();		
	}
	
	private Double parameterFactor;

	@Override
	public void calcBiddingParameter(AuctionMessage message) {
		
		// Teste auf freie Ressourcen und/oder teste Auslastung der DCs
				// Rejecte das Auktionsangebot oder whle eine BiddingStrategie in
				// Abhngigkeit der Ergebnisse aus:
				// Aggressiv(Greedy), Standard, Vorsichtig, Luxus

				// anteil der fixkosten am bestmglichen Einnahmeergebnis, bestimmen ber accounting
				double breakEvenThreshold = getStrategy().getCb().getCsp().getAccounting().calculateBreakEvenPercentage();

				// TODO auslesen aus der SlaManagerStrategie?
				double observingThreshold = 0.8d;
				double dangerThreshold = 0.9d;

				// calculate the actual mean load percentage for this csp over all its hosts 
				List<FederatedDatacenter> fedDcs = new ArrayList<FederatedDatacenter>(getStrategy().getCb().getCsp().getDcList());
				double meanLoadPercentageCsp = 0;
				for (int j = 0; j < fedDcs.size(); j++) {
					FederatedDatacenter fedDc = fedDcs.get(j);
					List<FederatedHost> fedHosts = fedDc.getHostList();
					double meanLoadPercentageDc = 0;
					for (int i = 0; i < fedHosts.size(); i++) {
						FederatedHost fedHost = fedHosts.get(i);
						double ramLoadPercentage = (double) (fedHost.getResources().getRam().getUsed())
								/ (double) fedHost.getResources().getRam().getTotal();
						double cpuLoadPercentage = (fedHost.getResources().getMips().getUsed())
								/ fedHost.getResources().getMips().getTotal();
						double loadPercentage = (ramLoadPercentage + cpuLoadPercentage) / 2;
						meanLoadPercentageDc = (meanLoadPercentageDc * i + loadPercentage) / (i + 1);
					}
					meanLoadPercentageCsp = (meanLoadPercentageCsp * j + meanLoadPercentageDc) / (j + 1);
				}
				Map<String, String> newBidMapEntry = new TreeMap<>();
				newBidMapEntry.put("Auction", String.valueOf(message.getCorrespondingAuction().getAuctionId()));
				newBidMapEntry.put("Load", String.valueOf(Math.round(meanLoadPercentageCsp *10000)/100));
				newBidMapEntry.put("First", String.valueOf(0d));
				newBidMapEntry.put("Second", String.valueOf(0d));
				
				//TODO get the parameter from xml
				if (meanLoadPercentageCsp < breakEvenThreshold + 0.15) {
					// greedy
					parameterFactor = 0.30d;
				} else {
					if (meanLoadPercentageCsp < observingThreshold) {
						// standard
						parameterFactor = 0.80d;
					} else {
						if (meanLoadPercentageCsp < dangerThreshold) {
							// cautious
							parameterFactor = 0.95d;
						}						
					}
				}
				
				if (meanLoadPercentageCsp < dangerThreshold && message.getRound() == 1){
					for (String string : message.getBiddingParameters().keySet()) {
						message.getBiddingParameters().put(string, message.getParticipant().getAccounting().getStrategy().getAccountingParameter().get(string));
					}	
					newBidMapEntry.put("First", String.valueOf(Math.round(parameterFactor *10000)/100));
					
				}		
				if (meanLoadPercentageCsp < dangerThreshold && message.getRound() >= 2){
					for (String string : message.getBiddingParameters().keySet()) {		
						Double parameter = Double.parseDouble(message.getParticipant().getAccounting().getStrategy().getAccountingParameter().get(string));
						message.getBiddingParameters().put(string, String.valueOf(parameter*parameterFactor));
					}
					newBidMapEntry.put("Second", String.valueOf(Math.round(parameterFactor *10000)/100));
				}
				
				if (meanLoadPercentageCsp >= dangerThreshold) {
					// luxus
					parameterFactor = 1.5d;
					for (String string : message.getBiddingParameters().keySet()) {		
						Double parameter = Double.parseDouble(message.getParticipant().getAccounting().getStrategy().getAccountingParameter().get(string));
						message.getBiddingParameters().put(string, String.valueOf(parameter*parameterFactor));
					}
					if(message.getRound() == 1)
						newBidMapEntry.put("First", String.valueOf(Math.round(parameterFactor *10000)/100));
					if(message.getRound() >= 2)
						newBidMapEntry.put("Second", String.valueOf(Math.round(parameterFactor *10000)/100));
				}				
				getStrategy().getAuctionManager().getAuctionBidMap().put(message.getCorrespondingAuction(), newBidMapEntry);
	}

}
