package com.materna.federatedcloudsim.utils;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

public class TraceShortener {

	private int maxLines = 8300;
	private int startTime = 1376314846;
	private int endTime = 1378906798;
	private List<String> inputPath;
	private List<String> outputPath;
	private String csvsplit;
	private int coresPos;
	private int ramUsagePos;
	private int cpuUsagePercentagePos;
	private int cpuUsagePos;
	private int ramTotalPos;
	private int tracetimePos;
	private int cpuCapacityPos;
	private boolean convertTimestamp;

	public TraceShortener() {
		/*
		 * number of lines to format/write (one line = one tick)
		 */
		maxLines = 8300;

		/*
		 * starttime: if one csv file starts earlier --> fill values with 0
		 * until starttime reached
		 */
		startTime = 1376314846;

		/*
		 * endtime: if one csv file ends before endtime --> fill values with 0
		 * until endtime reached (or maxlines)
		 */
		endTime = 1378906798;

		/*
		 * The split character of the csv files.
		 */
		csvsplit = ";";

		/*
		 * csv-positions of necessary values
		 * 
		 * cores: number of cpu cores ramUsage: currently used ram (KB)
		 * ramTotal: total (allocated/provisioned) ram cpuUsage: currently used
		 * cpu (mips-->MHz) cpuUsagePercentage: currently used cpu (%)
		 * cpuCapacity: total (allocated/provisioned) cpu (MHz) tracetime: time
		 * of each tick (if time is DateTime-format --> convert time to
		 * timestamp in ms since 1970)
		 */
		coresPos = 1;
		ramUsagePos = 6;
		ramTotalPos = 5;
		cpuUsagePercentagePos = 4;
		cpuUsagePos = 3;
		tracetimePos = 0;
		cpuCapacityPos = 2;

		/*
		 * if time is "date time" like "05.11.2015 00:00:00" --> convert into
		 * time of seconds from 1970
		 */
		convertTimestamp = true;

		/*
		 * Set the path to every workloadfolder.
		 */
		inputPath = new ArrayList<String>();
		inputPath.add("C:/Users/mpfahlbe/Desktop/FCS/experimente/workloads_andreas_neu/Materna-Trace1/");

		/*
		 * Set the output paths.
		 */
		outputPath = new ArrayList<String>();
		outputPath.add("C:/Users/mpfahlbe/Desktop/FCS/workloads/materna_short/Trace1/workloads");
	}

	public static void main(String[] args) throws NumberFormatException, ParseException, Exception {
		new TraceShortener().generate();
	}

	public void generate() throws NumberFormatException, ParseException, Exception {
		System.out.println("[STARTING] \t Converting workloads...");

		String writeline = "";
		BufferedReader br = null;
		BufferedReader br1 = null;

		int firstTraceTime = 0;
		int lastTraceTime = 0;
		String line = "";
		String[] logData;
		int j;
		int cores = 0;
		double ram = 0;
		double cpu = 0;
		DecimalFormat f = new DecimalFormat("#0.0000");
		double cpuSum;

		for (int ipIndex = 0; ipIndex < inputPath.size(); ipIndex++) {
			String inputFolder = inputPath.get(ipIndex);

			File wlFolder = new File(inputFolder);
			File[] wlFiles = wlFolder.listFiles();

			List<File> orderedFileList = new ArrayList<File>();
			for (int i = 0; i < wlFiles.length; i++)
				orderedFileList.add(wlFiles[i]);

			orderedFileList.sort((File o1, File o2) -> ((Integer) Integer
					.parseInt(o1.getName().substring(0, o1.getName().lastIndexOf(".")))).compareTo(
							((Integer) Integer.parseInt(o2.getName().substring(0, o2.getName().lastIndexOf("."))))));

			for (int i = 0; i < orderedFileList.size(); i++) {
				try {
					cpuSum = 0;
					cores = 0;
					ram = 0;

//					String csvFile = inputFolder + wlFiles[i].getName();
					String csvFile = inputFolder + orderedFileList.get(i).getName();
					br = new BufferedReader(new FileReader(csvFile));
					br.readLine(); // read first info line out

					br1 = new BufferedReader(new FileReader(csvFile));
					br1.readLine();

					String valueToParse = null;

					while ((line = br1.readLine()) != null) {
						logData = line.split(csvsplit);

						writeline = logData[coresPos].replaceAll(" ", "");
						writeline = writeline.replaceAll("\t", "");

						if (writeline.indexOf("\"") == -1)
							cores = Integer.parseInt(writeline);
						else if ((Integer.parseInt(writeline.substring(writeline.indexOf("\"") + 1,
								writeline.lastIndexOf("\"")))) > cores) {
							cores = Integer.parseInt(
									writeline.substring(writeline.indexOf("\"") + 1, writeline.lastIndexOf("\"")));
						}

						/*
						 * RAM
						 */
						if (logData[ramTotalPos].indexOf("\"") == -1)
							valueToParse = logData[ramTotalPos];
						else
							valueToParse = logData[ramTotalPos].substring(logData[ramTotalPos].indexOf("\"") + 1,
									logData[ramTotalPos].lastIndexOf("\""));

						if (Double.parseDouble(valueToParse) > ram)
							ram = Double.parseDouble(valueToParse);

						/*
						 * CPU
						 */
						if (logData[cpuCapacityPos].indexOf("\"") == -1)
							valueToParse = logData[cpuCapacityPos];
						else
							valueToParse = logData[cpuCapacityPos].substring(logData[cpuCapacityPos].indexOf("\"") + 1,
									logData[cpuCapacityPos].lastIndexOf("\""));

						if (Double.parseDouble(valueToParse) > cpu)
							cpu = Double.parseDouble(valueToParse);

						if (logData[cpuUsagePos].indexOf("\"") == -1)
							valueToParse = logData[cpuUsagePos];
						else
							valueToParse = logData[cpuUsagePos].substring(logData[cpuUsagePos].indexOf("\"") + 1,
									logData[cpuUsagePos].lastIndexOf("\""));

						cpuSum = cpuSum + Double.parseDouble(valueToParse) * 300;

						/*
						 * (last) TraceTime
						 */
						if (logData[tracetimePos].indexOf("\"") == -1)
							valueToParse = logData[tracetimePos];
						else
							valueToParse = logData[tracetimePos].substring(logData[tracetimePos].indexOf("\"") + 1,
									logData[tracetimePos].lastIndexOf("\""));

						lastTraceTime = getTraceTime(valueToParse);

					}

					File of = new File(outputPath.get(ipIndex));

					if (of.mkdirs())
						System.out.println("[RUNNING] \t Created output folder.");

//					try (Writer writer = new BufferedWriter(new OutputStreamWriter(
//							new FileOutputStream(outputPath.get(ipIndex) + "/" + (i + 1) + ".csv"), "utf-8"))) {

					try (Writer writer = new BufferedWriter(new OutputStreamWriter(
							new FileOutputStream(outputPath.get(ipIndex) + "/" + orderedFileList.get(i).getName()), "utf-8"))) {
						
						writer.write("#CPU Cores, RAM, cpuUsedSum;" + cores + ";" + ram + ";"
								+ (long) cpuSum + "\r");

						j = 1;
						while ((line = br.readLine()) != null && j < maxLines) {

							logData = line.split(csvsplit);
							if (j == 1) {

								String valueToParse2 = null;
								if (logData[tracetimePos].indexOf("\"") == -1)
									valueToParse2 = logData[tracetimePos];
								else
									valueToParse2 = logData[tracetimePos].substring(
											logData[tracetimePos].indexOf("\"") + 1,
											logData[tracetimePos].lastIndexOf("\""));
								firstTraceTime = getTraceTime(valueToParse2);
							}

							/*
							 * // trace starts later while (firstTraceTime >
							 * startTime) { writeline = j + ";0;0";
							 * writer.write(writeline + "\r"); firstTraceTime =
							 * firstTraceTime - 300; j++; }
							 */
							String valueToParseCPU = null;
							if (logData[cpuUsagePercentagePos].indexOf("\"") == -1)
								valueToParseCPU = logData[cpuUsagePercentagePos];
							else
								valueToParseCPU = logData[cpuUsagePercentagePos].substring(
										logData[cpuUsagePercentagePos].indexOf("\"") + 1,
										logData[cpuUsagePercentagePos].lastIndexOf("\""));

							String valueToParseRAM = null;
							if (logData[ramUsagePos].indexOf("\"") == -1)
								valueToParseRAM = logData[ramUsagePos];
							else
								valueToParseRAM = logData[ramUsagePos].substring(logData[ramUsagePos].indexOf("\"") + 1,
										logData[ramUsagePos].lastIndexOf("\""));
							writeline = j + ";"
									+ (f.format(Double.parseDouble(valueToParseCPU.replaceAll(",", ".")) / 100)
											.replaceAll(",", "."))
									+ ";" + f.format(Double.parseDouble(valueToParseRAM) / ram).replaceAll(",", ".");
							writeline.replaceAll(",", ".");
							writer.write(writeline + "\r");
							j++;
						}
						/*
						 * while (lastTraceTime < endTime && j < maxLines) {
						 * writeline = j + ";0;0"; writer.write(writeline +
						 * "\r"); lastTraceTime = lastTraceTime + 300; j++; }
						 */
						writer.close();

					}
				} catch (FileNotFoundException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}

			System.out.println("[RUNNING] \t Converted " + (inputPath.indexOf(inputFolder) + 1) + ". workloads");
		}

		System.out.println("\n[FINNISHED] \t Converted all workloads!");
	}

	private int getTraceTime(String dateTime) throws NumberFormatException, ParseException, Exception {
		int ret = 0;

		if (convertTimestamp)
			ret = Integer.parseInt(convertDateTimeToTimestamp(dateTime));
		else
			ret = Integer.parseInt(dateTime);

		return ret;
	}

	private String convertDateTimeToTimestamp(String datetime) throws ParseException, Exception {
		// converts the following date time into time of seconds from 1970
		// 05.11.2015 00:00:00
		DateFormat format = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss z", Locale.GERMAN);
		Date date;
		String newDatetime = null;

		try {
			date = format.parse(datetime + " GMT");
		} catch (Exception e) {
			date = format.parse(datetime + " GMT");
		}

		newDatetime = String.format("%d", date.getTime() / 1000);

		if (newDatetime == null)
			throw new Exception("Couldnt foramt DateTimeString");

		return newDatetime;
	}
}
