package com.materna.federatedcloudsim.utils;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class WorkloadFormatter {

	private boolean copyRenameOn;
	private boolean generateSwfOn;
	private boolean randomizeOn;

	private List<String> inputPath;
	private String outputPath;
	private String wlInputPath;
	private String swfFileName;
	private List<Integer> vmCountList;

	public double slaGoldValue;
	public double slaSilverValue;
	public double slaBronzeValue;

	int vmCountTotal = 1;

	public WorkloadFormatter() {

		/*
		 * Possible as single or first operation
		 */
		copyRenameOn = false;
		/*
		 * If true --> vms are randomly sorted. (not csp overlapping)
		 */
		randomizeOn = false;
		/*
		 * Possible as single operation or second operation after "copyRename"
		 */
		generateSwfOn = true;

		/*
		 * #####################################################################
		 * The following parameters must be set only, IF copyRenameOn == true:
		 * #####################################################################
		 */

		/*
		 * Set the path to every workloadfolder. The sequence is important:
		 * First Folder will be included first etc...so the first CSP will be
		 * initialized with the workloads wihtin the first path.
		 */
		inputPath = new ArrayList<String>();
		inputPath.add("C:/Users/mpfahlbe/Desktop/FCS/experimente/workloads/fastStorage_short_randomized/workloads/");
		inputPath.add("C:/Users/mpfahlbe/Desktop/FCS/experimente/workloads/rnd_short_randomized/merged/workloads/");
		// inputPath.add("C:/Users/mpfahlbe/Desktop/FCS/experimente/workloads/rnd_short_randomized/2013-7/workloads/");
		// inputPath.add("C:/Users/mpfahlbe/Desktop/FCS/experimente/workloads/rnd_short_randomized/2013-8/workloads/");
		// inputPath.add("C:/Users/mpfahlbe/Desktop/FCS/experimente/workloads/materna_short_randomized/Trace1/workloads/");
		// inputPath.add("C:/Users/mpfahlbe/Desktop/FCS/experimente/workloads/materna_short_randomized/Trace2/workloads/");

		/*
		 * #####################################################################
		 * The following parameters must be set only, IF generateSwfOn == true:
		 * #####################################################################
		 */

		/*
		 * The name of the swf-file which will be generated
		 */
		swfFileName = "2Csps_fst-random_rndMerged-random.swf";

		/*
		 * Set the percentage occurrence of each sla level (per csp)
		 */
		slaGoldValue = 0.2;
		slaSilverValue = 0.3;
		slaBronzeValue = 0.5;

		/*
		 * #####################################################################
		 * The following parameters must be set only, IF copyRenameOn == false
		 * AND generateSwf = true:
		 * #####################################################################
		 */

		/*
		 * Must be filled manually only, if "copyRenameOn == false"!
		 * 
		 * Contains the path to the workloadFolder.
		 */
		wlInputPath = "C:/Users/mpfahlbe/Desktop/FCS/experimente/workloads/2Csps_fst-random_rndMerged-random/workloads/";

		/*
		 * MUST BE FILLED MANUALLY ONLY, IF "copyRenameOn == false"! OR if you
		 * want a different deployment to CSPs (then each entry defines a csp
		 * (number of CSPs == vmCountList.size()), which takes
		 * vmCountList.get(i) vms)
		 * 
		 * Every entry contains the number of vms per csp (number of vms for
		 * each csp).
		 */
		vmCountList = new ArrayList<Integer>();
		vmCountList.add(1250);
		vmCountList.add(1500);
		// vmCountList.add(500);
		// vmCountList.add(500);
		// vmCountList.add(520);
		// vmCountList.add(527);

		/*
		 * #####################################################################
		 * General paramteres
		 * #####################################################################
		 */

		/*
		 * Empty destination folder. Workload-Folder: All directories will be
		 * created, if they don't exist.
		 * 
		 * The swf file will be created in this folder. All workloads will be
		 * created in: otputPath + /workloads
		 */
		outputPath = "C:/Users/mpfahlbe/Desktop/FCS/experimente/workloads/2Csps_fst-random_rndMerged-random/";

	}

	public void start() throws IOException {

		if (copyRenameOn && !randomizeOn)
			copyRenameWL();
		else if (copyRenameOn && randomizeOn)
			copyRenameWLRandomized();

		if (randomizeOn && copyRenameOn)
			randomize(null);
		else if (randomizeOn && !copyRenameOn)
			randomize(wlInputPath);

		if (generateSwfOn && copyRenameOn)
			generateSWF(null);
		else if (generateSwfOn && !copyRenameOn)
			generateSWF(wlInputPath);

		System.out.println("\n\nALL DONE!");
	}

	private void copyRenameWL() throws IOException {
		System.out.println("STARTING: Copy and rename every workload-file!");

		// incremented to rename every csv-file

		File outputFile = new File(outputPath + "workloads");

		// create all non existing directories
		if (outputFile.mkdirs())
			System.out.println("Created all nonexistend output directorys!");

		boolean countedManually = false;
		if (vmCountList.size() != 0)
			countedManually = true;

		// For every input path
		for (String inputPath : inputPath) {
			int vmCountFolder = 0;
			File wlFolder = new File(inputPath);
			List<File> orderedFileList = new ArrayList<File>();
			boolean notInt = false;

			for (File f : wlFolder.listFiles()) {
				orderedFileList.add(f);

				try {
					Integer.parseInt((f.getName().substring(0, f.getName().lastIndexOf("."))));
				} catch (NumberFormatException nfe) {
					notInt = true;
				}
			}

			// sorts the csv-input-files ascending
			if (!notInt)
				orderedFileList.sort((File o1,
						File o2) -> ((Integer) Integer
								.parseInt((o1.getName().substring(0, o1.getName().lastIndexOf(".")))))
										.compareTo(((Integer) Integer
												.parseInt(o2.getName().substring(0, o2.getName().lastIndexOf("."))))));

			// Copy and rename all Files in this directory
			for (File csVFile : orderedFileList) {
				Files.copy(csVFile.toPath(), outputFile.toPath().resolve(vmCountTotal++ + ".csv"));

				vmCountFolder++;
			}

			if (!countedManually)
				vmCountList.add(vmCountFolder);
		}

		System.out.println("FINISHED: Copied and renamed " + (vmCountTotal - 1) + " files!");
	}

	private void copyRenameWLRandomized() throws IOException {
		System.out.println("STARTING: Copy and rename every workload-file!");

		// incremented to rename every csv-file

		File outputFile = new File(outputPath + "workloads");

		// create all non existing directories
		if (outputFile.mkdirs())
			System.out.println("Created all nonexistend output directorys!");

		boolean countedManually = false;
		if (vmCountList.size() != 0)
			countedManually = true;

		// For every input path
		for (String inputPath : inputPath) {
			int vmCountFolder = 0;
			File wlFolder = new File(inputPath);
			File[] files = wlFolder.listFiles();
			Random random = new Random();

			List<File> orderedFileList = new ArrayList<File>();
			boolean notInt = false;
			for (File f : wlFolder.listFiles()) {
				orderedFileList.add(f);

				try {
					Integer.parseInt((f.getName().substring(0, f.getName().lastIndexOf("."))));
				} catch (NumberFormatException nfe) {
					notInt = true;
				}
			}

			// sorts the csv-input-files ascending
			if (!notInt)
				orderedFileList.sort((File o1,
						File o2) -> ((Integer) Integer
								.parseInt((o1.getName().substring(0, o1.getName().lastIndexOf(".")))))
										.compareTo(((Integer) Integer
												.parseInt(o2.getName().substring(0, o2.getName().lastIndexOf("."))))));

			for (int i = 0; i < files.length; i++) {
				int rnd = random.nextInt(files.length);
				File tmp = files[i];
				files[i] = files[rnd];
				files[rnd] = tmp;
			}

			// Copy and rename all Files in this directory
			for (File csVFile : files) {
				Files.copy(csVFile.toPath(), outputFile.toPath().resolve(vmCountTotal++ + ".csv"));

				vmCountFolder++;
			}

			if (!countedManually)
				vmCountList.add(vmCountFolder);
		}

		System.out.println("FINISHED: Copied and renamed " + (vmCountTotal - 1) + " files!");
	}

	private void randomize(String wlInputPath) throws IOException {
		System.out.println("STARTING: Randomizing!");

		if (wlInputPath == null || wlInputPath.equals(""))
			wlInputPath = outputPath + "/workloads/";

		File outputFile = new File(outputPath + "workloads");
		outputFile.mkdirs();
		File wlFolder = new File(wlInputPath);
		File[] files = wlFolder.listFiles();
		int rndCount = 1;
		Random random = new Random();
		int lastEnd = 0;

		List<File> orderedFileList = new ArrayList<File>();
		boolean notInt = false;

		for (File f : wlFolder.listFiles()) {
			orderedFileList.add(f);

			try {
				Integer.parseInt((f.getName().substring(0, f.getName().lastIndexOf("."))));
			} catch (NumberFormatException nfe) {
				notInt = true;
			}
		}

		// sorts the csv-input-files ascending
		if (!notInt)
			orderedFileList.sort((File o1, File o2) -> ((Integer) Integer
					.parseInt((o1.getName().substring(0, o1.getName().lastIndexOf("."))))).compareTo(
							((Integer) Integer.parseInt(o2.getName().substring(0, o2.getName().lastIndexOf("."))))));

		files = new File[orderedFileList.size()];
		for (int i = 0; i < orderedFileList.size(); i++)
			files[i] = orderedFileList.get(i);

		for (int j = 0; j < vmCountList.size(); j++) {
			int rnd = random.nextInt(vmCountList.get(j));
			rnd += lastEnd;

			for (int i = lastEnd; i < (lastEnd + vmCountList.get(j)); i++) {
				rnd = random.nextInt(vmCountList.get(j));
				rnd += lastEnd;

				File tmp = files[i];
				files[i] = files[rnd];
				files[rnd] = tmp;
			}

			lastEnd += vmCountList.get(j);
		}

		for (File csvFile : files) {
			Files.copy(csvFile.toPath(), outputFile.toPath().resolve(rndCount + ".csv"));
			File outputFile2 = new File(outputPath + "matching");
			outputFile2.mkdirs();
			Files.copy(csvFile.toPath(), outputFile2.toPath().resolve(
					csvFile.getName().substring(0, csvFile.getName().lastIndexOf(".")) + "_" + rndCount + ".csv"));

			rndCount++;
		}
	}

	private void generateSWF(String wlInputPath) {

		System.out.println("STARTING: Creating SWF-File!");

		if (wlInputPath == null || wlInputPath.equals(""))
			wlInputPath = outputPath + "/workloads/";

		String writeline = "";
		BufferedReader br = null;
		String line = "";
		String csvsplit = ";";
		int gold_sla = 0;
		int silver_sla = 0;
		int bronze_sla = 0;
		int rnd_val = 0;
		String[] logData;
		Random rn = new Random();
		boolean foundSlaLevel = false;

		try {
			Writer writer = new BufferedWriter(
					new OutputStreamWriter(new FileOutputStream(outputPath + swfFileName), "utf-8"));
			int vmCountSwf = 1;

			for (int j = 0; j < vmCountList.size(); j++) {

				for (int i = 1; i <= vmCountList.get(j); i++) {
					rnd_val = rn.nextInt(3) + 1;
					foundSlaLevel = false;

					while (!foundSlaLevel) {
						switch (rnd_val) {
						case 1:
							if (gold_sla < slaGoldValue * vmCountList.get(j)) {
								foundSlaLevel = true;
								gold_sla++;
							} else {
								rnd_val = (rnd_val + 1) % 4;
							}
							break;
						case 2:
							if (silver_sla < slaSilverValue * vmCountList.get(j)) {
								foundSlaLevel = true;
								silver_sla++;
							} else {
								rnd_val = (rnd_val + 1) % 4;
							}
							break;
						case 3:
							if (bronze_sla < slaBronzeValue * vmCountList.get(j)) {
								foundSlaLevel = true;
								bronze_sla++;
							} else {
								rnd_val = (rnd_val + 1) % 4;
							}
							break;
						}
					}

					String csvFile = wlInputPath + vmCountSwf + ".csv";
					br = new BufferedReader(new FileReader(csvFile));
					line = br.readLine(); // read first info line out
					logData = line.split(csvsplit);
					writeline = vmCountSwf + // 1 JobNR
							" 0" + // 2 Submit Time
							" 0" + " " + // 3 Wait Time
							" 0" + " " + // 4 Runtime
							logData[1] + // 5 NUM Proc
							" " + " 0" + " " + // 6 CPU Time
							// ((long) (Double.parseDouble(logData[2]))) * 1024
							// * 1024 + // 7
							// // MEM
							((long) (Double.parseDouble(logData[2]))) + // 7
							// MEM
							// Used
							" " + logData[1] + " " // 8 CPU Requiered
							+ " 0" + " " + // 9 Runtime requiered
							// ((long) (Double.parseDouble(logData[2]))) * 1024
							// * 1024 + // 10
							// // RAM
							((long) (Double.parseDouble(logData[2]))) + // 10
							// RAM
							// requiered
							" " + rnd_val + // 11 the SLA-Level
							" " + j // user-id (uid)
							+ " 1 1 -1 1 -1 -1"; // Irrelavant values
					writer.write(writeline + "\n");

					vmCountSwf++;
				}

				gold_sla = 0;
				silver_sla = 0;
				bronze_sla = 0;
			}
			writer.close();

			System.out.println("FINISHED: Generated SWF File!");

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) throws IOException {
		new WorkloadFormatter().start();
	}

}
